# Generated by Django 1.10.6 on 2017-04-10 14:29

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("order", "0014_auto_20161209_2045"),
    ]

    operations = [
        migrations.AlterField(
            model_name="order_item",
            name="size",
            field=models.CharField(
                blank=True,
                choices=[("", ""), ("R", "Regular"), ("L", "Large")],
                max_length=1,
                null=True,
                verbose_name="size",
            ),
        ),
    ]



from django.contrib import admin

from souschef.order.models import (
    Order,
    Order_item,
    OrderStatusChange,
)


def make_delivered(modeladmin, request, queryset):
    queryset.update(status="D")


make_delivered.short_description = "Mark selected orders as delivered"


class OrderItemInline(admin.TabularInline):
    model = Order_item


class OrderItemAdmin(admin.ModelAdmin):
    search_fields = ["order__id"]
    list_display = (
        "order",
        "order_item_type",
        "component_group",
        "total_quantity",
        "billable_flag",
        "price",
    )


class OrderAdmin(admin.ModelAdmin):
    search_fields = ["client__member__lastname", "client__member__firstname"]
    list_filter = ("status", "delivery_date")
    list_display = (
        "id",
        "client",
        "status",
        "price",
        "delivery_date",
        "creation_date",
    )
    actions = [make_delivered]
    inlines = [OrderItemInline]


admin.site.register(Order, OrderAdmin)
admin.site.register(Order_item, OrderItemAdmin)
admin.site.register(OrderStatusChange)



from django.core.management import call_command
from django.core.management.base import BaseCommand

from souschef.member.factories import ClientFactory


class Command(BaseCommand):
    help = "Creates a happy bunch of clients without orders"

    def handle(self, *args, **options):
        # Load fixtures for the routes
        fixture_filename = "routes.json"
        call_command("loaddata", fixture_filename)

        client = ClientFactory.create_batch(10)
        self.stdout.write(
            self.style.SUCCESS('Successfully created client "%s"' % client)
        )



from django.db import models
from django.utils.translation import ugettext_lazy as _

COMPONENT_GROUP_CHOICES = (
    ("main_dish", _("Main Dish")),
    ("dessert", _("Dessert")),
    ("diabetic", _("Diabetic")),
    ("fruit_salad", _("Fruit Salad")),
    ("green_salad", _("Green Salad")),
    ("pudding", _("Pudding")),
    ("compote", _("Compote")),
    ("sides", _("Sides")),
)

COMPONENT_GROUP_CHOICES_MAIN_DISH = COMPONENT_GROUP_CHOICES[0][0]
COMPONENT_GROUP_CHOICES_SIDES = COMPONENT_GROUP_CHOICES[7][0]

INGREDIENT_GROUP_CHOICES = (
    ("meat", _("Meat")),
    ("dairy", _("Dairy")),
    ("fish", _("Fish")),
    ("seafood", _("Seafood")),
    ("veggies_and_fruits", _("Veggies and fruits")),
    ("legumineuse", _("Legumineuse")),
    ("grains", _("Grains")),
    ("fresh_herbs", _("Fresh herbs")),
    ("spices", _("Spices")),
    ("dry_and_canned_goods", _("Dry and canned goods")),
    ("oils_and_sauces", _("Oils and sauces")),
)

RESTRICTED_ITEM_GROUP_CHOICES = (
    ("meat", _("Meat")),
    ("vegetables", _("Vegetables")),
    ("seafood", _("Seafood")),
    ("seeds and nuts", _("Seeds and nuts")),
    ("other", _("Other")),
)


class Ingredient(models.Model):
    class Meta:
        verbose_name_plural = _("ingredients")

    # Information about the ingredient (in the recipe of a component)
    name = models.CharField(
        max_length=50,
        verbose_name=_("name"),
        unique=True,
    )

    description = models.TextField(
        verbose_name=_("description"),
        blank=True,
        null=True,
    )

    ingredient_group = models.CharField(
        max_length=100,
        choices=INGREDIENT_GROUP_CHOICES,
        verbose_name=_("ingredient group"),
    )

    def __str__(self):
        return self.name


class Component(models.Model):
    class Meta:
        verbose_name_plural = _("components (Dishes and recipes)")

    # Meal component (ex. main dish, vegetable, seasonal) information
    name = models.CharField(
        max_length=50,
        verbose_name=_("name"),
        unique=True,
    )

    description = models.TextField(
        verbose_name=_("description"),
        blank=True,
        null=True,
    )

    component_group = models.CharField(
        max_length=100,
        choices=COMPONENT_GROUP_CHOICES,
        verbose_name=_("component group"),
    )

    def __str__(self):
        return self.name

    @staticmethod
    def get_day_ingredients(component_id, delivery_date):
        """Returns a list of the actual ingredients
        of a component for the delivery date."""
        q = Component_ingredient.objects.select_related("ingredient").filter(
            component__id=component_id, date=delivery_date
        )
        return [ci.ingredient for ci in q]

    @staticmethod
    def get_recipe_ingredients(component_id):
        """Returns a list of the ingredients in the recipe of a component."""
        q = Component_ingredient.objects.select_related("ingredient").filter(
            component__id=component_id, date=None
        )
        return [ci.ingredient for ci in q]


class Component_ingredient(models.Model):
    component = models.ForeignKey(
        "meal.Component",
        verbose_name=_("component"),
        related_name="+",
        on_delete=models.CASCADE,
    )

    ingredient = models.ForeignKey(
        "meal.Ingredient",
        verbose_name=_("ingredient"),
        related_name="+",
        on_delete=models.CASCADE,
    )

    date = models.DateField(
        verbose_name=_("date"),
        blank=True,
        null=True,
    )

    def __str__(self):
        if self.date:
            return (
                f"<{self.component.name}> includes <{self.ingredient.name}> on "
                f"<{self.date}>"
            )
        else:
            return f"<{self.component.name}> contains <{self.ingredient.name}>"


class Restricted_item(models.Model):
    class Meta:
        verbose_name_plural = _("restricted items (Restriction categories)")

    # Information about restricted item categories that some clients never eat
    #   for allergy or other reasons (ex. Gluten, Nuts, Pork)
    name = models.CharField(max_length=50, verbose_name=_("name"))

    description = models.TextField(
        verbose_name=_("description"),
        blank=True,
        null=True,
    )

    restricted_item_group = models.CharField(
        max_length=100,
        choices=RESTRICTED_ITEM_GROUP_CHOICES,
        verbose_name=_("restricted item group"),
    )

    ingredients = models.ManyToManyField("meal.Ingredient", through="Incompatibility")

    def __str__(self):
        return self.name


class Incompatibility(models.Model):
    restricted_item = models.ForeignKey(
        "meal.Restricted_item",
        verbose_name=_("restricted item"),
        related_name="+",
        on_delete=models.CASCADE,
    )

    ingredient = models.ForeignKey(
        "meal.Ingredient",
        verbose_name=_("ingredient"),
        related_name="+",
        on_delete=models.CASCADE,
    )

    def __str__(self):
        return f"{self.restricted_item.name} <clash> {self.ingredient.name}"


class Menu(models.Model):
    class Meta:
        verbose_name_plural = _("menus")

    # Menu information for a specific date
    date = models.DateField(verbose_name=_("date"))

    components = models.ManyToManyField("meal.Component", through="Menu_component")

    def __str__(self):
        return f"Menu for {str(self.date)}"

    @staticmethod
    def create_menu_and_components(menu_date, dish_names):
        """Create the menu and its components for the date.

        Parameters:
          menu_date : date on which menu will apply
          dish_names : a list of names of dishes included in the menu

        Behavior:
          - if Menu(s) already exists for the date, only one is kept and
            all its menu components are replaced by the new ones

        Raises :
          - Exception if dish does not exist
          - Exception if more that one dish per component_group
          - Exception if no dish for COMPONENT_GROUP_CHOICES_MAIN_DISH

        Returns:
          Number of menu components created.
        """

        num_menu_comp_created = 0
        component_groups = {}
        menus = Menu.objects.filter(date=menu_date)
        if not menus:
            menu = Menu(date=menu_date)
            menu.save()
        else:
            # menu(s) existed, must flush their menu_components
            Menu_component.objects.filter(menu__in=menus).delete()
            # flush extra menus
            for menu in menus[1:]:
                Menu.objects.filter(id=menu.id).delete()
            # reuse the first one
            menu = menus[0]
        for dish_name in dish_names:
            components = Component.objects.filter(name=dish_name)
            if not components:
                raise Exception("Component ", dish_name, " does not exist")
            component = components[0]
            if component_groups.get(component.component_group):
                raise Exception(
                    "Menu can only have one dish for component group "
                    + component.component_group
                )
            component_groups[component.component_group] = dish_name
            menu_component = Menu_component(menu=menu, component=component)
            menu_component.save()
            num_menu_comp_created += 1
        # END FOR
        if not component_groups.get(COMPONENT_GROUP_CHOICES_MAIN_DISH):
            raise Exception(
                "Menu must include one dish for component group "
                + COMPONENT_GROUP_CHOICES_MAIN_DISH
            )
        return num_menu_comp_created


class Menu_component(models.Model):
    menu = models.ForeignKey(
        "meal.Menu", verbose_name=_("menu"), related_name="+", on_delete=models.CASCADE
    )

    component = models.ForeignKey(
        "meal.Component",
        verbose_name=_("component"),
        related_name="+",
        on_delete=models.CASCADE,
    )

    def __str__(self):
        return f"On {str(self.menu.date)} <menu includes> {self.component.name}"



# Generated by Django 1.10 on 2017-01-16 14:31

from django.db import migrations, models


def set_initial_categories(apps, schema_editor):
    # set NoteCategory initial
    NoteCategory = apps.get_model("note", "NoteCategory")
    NoteCategory.objects.create(name="Follow-up needed")
    NoteCategory.objects.create(name="No Charge")
    NoteCategory.objects.create(name="Orders/Cancellations")
    NoteCategory.objects.create(name="Comments (no actions needed)")
    NoteCategory.objects.create(name="Kitchen Comments")
    NoteCategory.objects.create(name="Stay late notes")
    NoteCategory.objects.create(name="Payment reception")


class Migration(migrations.Migration):
    dependencies = [
        ("note", "0006_change_priority_field"),
    ]

    operations = [
        migrations.CreateModel(
            name="NoteCategory",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=150, verbose_name="Name")),
            ],
            options={"ordering": ("name",), "verbose_name_plural": "Note categories"},
        ),
        migrations.RunPython(set_initial_categories),
    ]



# Generated by Django 1.11.1 on 2017-05-26 17:55

from datetime import date

import annoying.fields
import django.db.models.deletion
from django.core.exceptions import ObjectDoesNotExist
from django.db import migrations, models


def migrate_relationships(apps, schema_editor):
    """
    Referencing --> Relationship
    EmergencyContact --> Relationship
    """
    Relationship = apps.get_model("member", "Relationship")
    Referencing = apps.get_model("member", "Referencing")
    EmergencyContact = apps.get_model("member", "EmergencyContact")

    from souschef.member.models import Relationship as _ref_Relationship  # noqa

    for ref in Referencing.objects.all().order_by("pk"):
        print(
            f"Converting member.Referencing #{ref.pk} to member.Relationship..."
        )
        # Attention: the Rel object may exist because a member can be both Ref and Emgc
        try:
            rel = Relationship.objects.get(member=ref.referent, client=ref.client)
        except ObjectDoesNotExist:
            rel = Relationship(member=ref.referent, client=ref.client, type=[])

        if _ref_Relationship.REFERENT not in rel.type:
            rel.type.append(_ref_Relationship.REFERENT)
        rel.nature = rel.nature or "??? (migration 0032)"
        rel.extra_fields = rel.extra_fields or {}
        rel.extra_fields["referral_date"] = ref.date
        rel.extra_fields["referral_reason"] = ref.referral_reason
        rel.save()

    for emgc in EmergencyContact.objects.all().order_by("pk"):
        print(
            f"Converting member.EmergencyContact #{emgc.pk} to member.Relationship..."
        )
        # Attention: the Rel object may exist because a member can be both Ref and Emgc
        try:
            rel = Relationship.objects.get(member=emgc.member, client=emgc.client)
        except ObjectDoesNotExist:
            rel = Relationship(member=emgc.member, client=emgc.client, type=[])

        if _ref_Relationship.EMERGENCY not in rel.type:
            rel.type.append(_ref_Relationship.EMERGENCY)
        rel.nature = emgc.relationship or rel.nature or "??? (migration 0032)"
        rel.save()

    print(
        "All converted. Deleting all member.Referencing and "
        "member.EmergencyContact objects..."
    )
    Referencing.objects.all().delete()
    EmergencyContact.objects.all().delete()


def reverse_migrate_relationships(apps, schema_editor):
    """
    Relationship --> Referencing / Emergency Contact
    """
    Relationship = apps.get_model("member", "Relationship")
    Referencing = apps.get_model("member", "Referencing")
    EmergencyContact = apps.get_model("member", "EmergencyContact")

    from souschef.member.models import Relationship as _ref_Relationship  # noqa

    for rel in Relationship.objects.all().order_by("pk"):
        if _ref_Relationship.REFERENT in rel.type:
            print(
                f"Reversing member.Relationship #{rel.pk} to member.Referencing..."
            )
            try:
                ref = Referencing.objects.get(referent=rel.member, client=rel.client)
            except Referencing.DoesNotExist:
                ref = Referencing(
                    referent=rel.member,
                    client=rel.client,
                    referral_reason="??? (reverse migration 0032)",
                    date=date(1970, 1, 1),
                )

            if isinstance(rel.extra_fields, dict):
                ref.referral_reason = rel.extra_fields.get(
                    "referral_reason", ref.referral_reason
                )
                ref.date = rel.extra_fields.get("referral_date", ref.date)
            ref.save()

        if _ref_Relationship.EMERGENCY in rel.type:
            print(
                f"Reversing member.Relationship #{rel.pk} to "
                "member.EmergencyContact..."
            )
            try:
                emgc = EmergencyContact.objects.get(
                    member=rel.member, client=rel.client
                )
            except EmergencyContact.DoesNotExist:
                emgc = EmergencyContact(member=rel.member, client=rel.client)
            emgc.relationship = rel.nature
            emgc.save()

    print("All reversed. Deleting all member.Relationship objects...")
    Relationship.objects.all().delete()


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0031_client_option_allow_reverse_relation"),
    ]

    operations = [
        # Add Relationship
        migrations.CreateModel(
            name="Relationship",
            options={"verbose_name_plural": "relationships"},
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("nature", models.CharField(max_length=100)),
                ("type", annoying.fields.JSONField(default=[])),
                ("extra_fields", annoying.fields.JSONField(default={})),
                ("remark", models.TextField()),
                (
                    "client",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="member.Client"
                    ),
                ),
                (
                    "member",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="member.Member"
                    ),
                ),
            ],
        ),
        migrations.AlterUniqueTogether(
            name="relationship",
            unique_together={("client", "member")},
        ),
        # Migrate
        migrations.RunPython(migrate_relationships, reverse_migrate_relationships),
        # Delete Referencing and EmergencyContact
        migrations.AlterUniqueTogether(
            name="emergencycontact",
            unique_together=set(),
        ),
        migrations.RemoveField(
            model_name="emergencycontact",
            name="client",
        ),
        migrations.RemoveField(
            model_name="emergencycontact",
            name="member",
        ),
        migrations.RemoveField(
            model_name="referencing",
            name="client",
        ),
        migrations.RemoveField(
            model_name="referencing",
            name="referent",
        ),
        migrations.RemoveField(
            model_name="client",
            name="emergency_contacts",
        ),
        migrations.DeleteModel(
            name="EmergencyContact",
        ),
        migrations.DeleteModel(
            name="Referencing",
        ),
    ]



import collections
import copy

from django.contrib import messages
from django.contrib.auth.mixins import (
    LoginRequiredMixin,
    PermissionRequiredMixin,
)
from django.db.models import (
    Count,
    Prefetch,
    Q,
)
from django.http import HttpResponseRedirect
from django.urls import reverse_lazy
from django.utils.translation import string_concat
from django.utils.translation import ugettext_lazy as _
from django.views import generic

from souschef.billing.models import (
    Billing,
    BillingFilter,
)
from souschef.member.models import Client
from souschef.order.models import (
    DeliveredOrdersByMonth,
    Order,
    Order_item,
)


class BillingList(LoginRequiredMixin, PermissionRequiredMixin, generic.ListView):
    # Display the billing list
    context_object_name = "billings"
    model = Billing
    permission_required = "sous_chef.read"
    template_name = "billing/list.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        uf = BillingFilter(self.request.GET, queryset=self.get_queryset())
        context["filter"] = uf

        return context

    def get_queryset(self):
        uf = BillingFilter(self.request.GET)
        return uf.qs.annotate(Count("orders", distinct=True))


class BillingCreate(LoginRequiredMixin, PermissionRequiredMixin, generic.CreateView):
    # View to create the billing
    context_object_name = "billing"
    model = Billing
    permission_required = "sous_chef.edit"

    def get(self, request):
        date = self.request.GET.get("delivery_date", "-")
        date = date if date != "" else "-"
        year, month = date.split("-")

        if year == "" or month == "":
            messages.add_message(
                self.request,
                messages.ERROR,
                _(
                    "You must select a period to create a billing and click in "
                    "the first step 'Preview'"
                ),
            )
            return HttpResponseRedirect(reverse_lazy("billing:add"))

        billing = Billing.objects.billing_create_new(year, month)

        messages.add_message(
            self.request,
            messages.SUCCESS,
            _(
                "The billing with the identifier #%s "
                "has been successfully created." % billing.id
            ),
        )
        return HttpResponseRedirect(reverse_lazy("billing:list"))


class BillingAdd(LoginRequiredMixin, PermissionRequiredMixin, generic.ListView):
    context_object_name = "orders"
    model = Order
    paginate_by = 20
    permission_required = "sous_chef.edit"
    template_name = "billing/add.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        uf = DeliveredOrdersByMonth(self.request.GET, queryset=self.get_queryset())
        context["filter"] = uf
        text = ""
        count = 0
        for getVariable in self.request.GET:
            if getVariable == "page":
                continue
            for getValue in self.request.GET.getlist(getVariable):
                if count == 0:
                    text += "?" + getVariable + "=" + getValue
                else:
                    text += "&" + getVariable + "=" + getValue
                count += 1

        text = text + "?" if count == 0 else text + "&"
        context["get"] = text

        return context

    def get_queryset(self):
        uf = DeliveredOrdersByMonth(self.request.GET)
        return (
            uf.qs.select_related("client__member")
            .only(
                "status",
                "delivery_date",
                "client__member__firstname",
                "client__member__lastname",
            )
            .prefetch_related(
                Prefetch(
                    "orders",
                    queryset=Order_item.objects.all().only(
                        "order__id", "price", "billable_flag"
                    ),
                )
            )
        )


class BillingSummaryView(
    LoginRequiredMixin, PermissionRequiredMixin, generic.DetailView
):
    # Display summary of billing
    model = Billing
    permission_required = "sous_chef.read"
    template_name = "billing/view.html"
    context_object_name = "billing"
    queryset = Billing.objects.prefetch_related(
        Prefetch(
            "orders",
            queryset=Order.objects.all()
            .select_related("client__member")
            .only(
                "client__member__firstname",
                "client__member__lastname",
                "client__rate_type",
                "client__billing_payment_type",
            )
            .prefetch_related(
                Prefetch(
                    "orders",
                    queryset=Order_item.objects.all().only(
                        "order__id",
                        "price",
                        "billable_flag",
                        "size",
                        "component_group",
                        "total_quantity",
                    ),
                )
            ),
        )
    )

    def get_template_names(self):
        if self.request.method == "GET" and self.request.GET.get("print"):
            return ["billing/print_summary.html"]
        else:
            return super().get_template_names()

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        billing = self.object

        # generate a summary
        zero_statistics = {
            "total_main_dishes": {"R": 0, "L": 0},
            "total_billable_sides": 0,
            "total_amount": 0,
            "total_deliveries": 0,
        }
        # target dict
        summary = copy.deepcopy(zero_statistics)
        summary["payment_types_dict"] = collections.defaultdict(
            lambda: dict(clients=[], **copy.deepcopy(zero_statistics))
        )
        for client, client_summary in billing.summary.items():
            t = client.billing_payment_type
            summary["payment_types_dict"][t]["total_main_dishes"][
                "R"
            ] += client_summary["total_main_dishes"]["R"]
            summary["payment_types_dict"][t]["total_main_dishes"][
                "L"
            ] += client_summary["total_main_dishes"]["L"]
            summary["payment_types_dict"][t]["total_billable_sides"] += client_summary[
                "total_billable_sides"
            ]
            summary["payment_types_dict"][t]["total_amount"] += client_summary[
                "total_amount"
            ]
            nb_deliveries = client.number_of_deliveries_in_month(
                billing.billing_year, billing.billing_month
            )
            summary["payment_types_dict"][t]["clients"].append(
                {
                    "id": client.id,
                    "firstname": client.member.firstname,
                    "lastname": client.member.lastname,
                    "payment_type": client.get_billing_payment_type_display()
                    if (client.billing_payment_type is not None)
                    else "",
                    "rate_type": client.get_rate_type_display()
                    if (client.rate_type != "default")
                    else "",
                    "total_main_dishes": client_summary["total_main_dishes"],
                    "total_billable_sides": client_summary["total_billable_sides"],
                    "total_amount": client_summary["total_amount"],
                    "delivery_type_verbose": client.delivery_type_verbose,
                    "number_of_deliveries_in_month": nb_deliveries,
                }
            )
            summary["total_main_dishes"]["R"] += client_summary["total_main_dishes"][
                "R"
            ]
            summary["total_main_dishes"]["L"] += client_summary["total_main_dishes"][
                "L"
            ]
            summary["total_billable_sides"] += client_summary["total_billable_sides"]
            summary["total_amount"] += client_summary["total_amount"]
            summary["total_deliveries"] += nb_deliveries

        # sort clients in each payment type group
        for _payment_type, statistics in summary["payment_types_dict"].items():
            statistics["clients"].sort(key=lambda c: (c["lastname"], c["firstname"]))

        # reorder the display for supported & non-supported payment types
        summary["payment_types"] = sorted(
            summary["payment_types_dict"].items(),
            key=lambda tup: {
                " ": 0,  # 0th position
                "credit": 1,  # 1st position
                "eft": 2,  # 2nd position
                "3rd": 3,  # 3rd position
            }.get(tup[0], 99),  # last position(s)
        )

        context["summary"] = summary

        # Throw a warning if there's any main_dish order with size=None.
        q = Order_item.objects.filter(
            Q(order__in=billing.orders.all())
            & (Q(size__isnull=True) | Q(size=""))
            & Q(component_group="main_dish")
        )
        if q.exists():
            size_none_orders_info = list(
                q.values_list(
                    "order__id",
                    "order__client__member__firstname",
                    "order__client__member__lastname",
                )
            )
            formatted_htmls = ['<ul class="ui list">']
            for i, f, l in size_none_orders_info:  # noqa: E741 (no idea what these are)
                formatted_htmls.append(
                    f'<li><a href="{Order(id=i).get_absolute_url()}" target="_blank">'
                    f"#{i} ({f} {l})"
                    "</a></li>"
                )
            formatted_htmls.append("</ul>")
            formatted_html = "".join(formatted_htmls)
            messages.add_message(
                self.request,
                messages.WARNING,
                string_concat(
                    _(
                        'Warning: the order(s) below have not set a "size" '
                        "for main dish and thus have been excluded in "
                        '"Total Main Dishes" column.'
                    ),
                    "<br/>",
                    formatted_html,
                ),
            )
        return context


class BillingOrdersView(
    LoginRequiredMixin, PermissionRequiredMixin, generic.DetailView
):
    # Display orders detail of billing
    model = Billing
    permission_required = "sous_chef.read"
    template_name = "billing/view_orders.html"
    context_object_name = "billing"

    queryset = Billing.objects.prefetch_related(
        Prefetch(
            "orders",
            queryset=Order.objects.all()
            .select_related("client__member")
            .only(
                "status",
                "delivery_date",
                "client__member__firstname",
                "client__member__lastname",
            )
            .prefetch_related(
                Prefetch(
                    "orders",
                    queryset=Order_item.objects.all().only(
                        "order__id", "price", "billable_flag"
                    ),
                )
            ),
        )
    )

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        if self.request.GET.get("client"):
            # has ?client=client_id
            client_id = int(self.request.GET["client"])
            orders = self.object.orders.filter(client__id=client_id)
            context["orders"] = orders
            context["client"] = Client.objects.get(id=client_id)
        else:
            context["orders"] = self.object.orders.all()

        context["total_amount"] = sum(map(lambda o: o.price, context["orders"]))
        context["clients"] = list(
            set(map(lambda o: o.client, self.object.orders.all()))
        )
        return context


class BillingDelete(LoginRequiredMixin, PermissionRequiredMixin, generic.DeleteView):
    model = Billing
    permission_required = "sous_chef.edit"
    template_name = "billing_confirm_delete.html"

    def get_success_url(self):
        # 'next' parameter should always be included in POST'ed URL.
        return self.request.GET.get("next") or self.request.POST["next"]



from django import forms
from django.utils.translation import ugettext_lazy as _
from extra_views import InlineFormSet

from souschef.meal.models import (
    COMPONENT_GROUP_CHOICES,
    COMPONENT_GROUP_CHOICES_SIDES,
)
from souschef.member.models import Client
from souschef.order.models import (
    SIZE_CHOICES,
    Order_item,
    OrderStatusChange,
)


class CreateOrderItem(InlineFormSet):
    model = Order_item
    extra = 1
    fields = "__all__"


class UpdateOrderItem(CreateOrderItem):
    extra = 0


class CreateOrdersBatchForm(forms.Form):
    def __init__(self, *args, **kwargs):
        if "delivery_dates" in kwargs:
            delivery_dates = kwargs["delivery_dates"]
            del kwargs["delivery_dates"]
        else:
            delivery_dates = None
        super().__init__(*args, **kwargs)

        if delivery_dates:
            for d in delivery_dates:
                self.fields[f"size_{d}"] = forms.ChoiceField(
                    choices=SIZE_CHOICES,
                    widget=forms.Select(attrs={"class": ""}),
                    required=True,
                )
                self.fields[f"delivery_{d}"] = forms.BooleanField(
                    required=False
                )
                self.fields[f"pickup_{d}"] = forms.BooleanField(required=False)
                self.fields[f"visit_{d}"] = forms.BooleanField(required=False)

                for meal, _meal_translation in COMPONENT_GROUP_CHOICES:
                    if meal is COMPONENT_GROUP_CHOICES_SIDES:
                        continue  # don't put "sides" on the form
                    self.fields[f"{meal}_{d}_quantity"] = forms.IntegerField(
                        min_value=0, required=True
                    )

    client = forms.ModelChoiceField(
        required=True,
        label=_("Client"),
        widget=forms.Select(attrs={"class": "ui search dropdown"}),
        queryset=Client.active.select_related("member")
        .only("member__firstname", "member__lastname")
        .order_by("member__lastname"),
    )

    delivery_dates = forms.CharField(
        required=True, label=_("Delivery dates"), max_length=200
    )

    override_dates = forms.CharField(
        required=False, label=_("Override dates"), max_length=200
    )

    is_submit = forms.IntegerField(required=True, label=_("Is form submit"))

    def clean_is_submit(self):
        is_submit = self.cleaned_data["is_submit"]
        if is_submit != 1:
            # prevents form submit and force a form refresh
            raise forms.ValidationError(_("This field must be 1 to submit the form."))
        return is_submit

    def clean(self):
        """
        For each delivery date, the quantities and options should
        not be all empty. Refs #803.
        """
        cleaned_data = super().clean()
        delivery_dates_str = cleaned_data.get("delivery_dates")
        delivery_dates = []
        if delivery_dates_str:
            delivery_dates = delivery_dates_str.split("|")
        fields_not_null_check = [
            ("delivery_{}", lambda x: x is True),
            ("pickup_{}", lambda x: x is True),
            ("visit_{}", lambda x: x is True),
        ]
        for meal, _meal_translation in COMPONENT_GROUP_CHOICES:
            if meal is COMPONENT_GROUP_CHOICES_SIDES:
                continue  # "sides" not in the form
            else:
                fields_not_null_check.append(
                    ("%s_{}_quantity" % meal, lambda x: x and x > 0)
                )
        for delivery_date in delivery_dates:
            if all(
                [
                    bool(
                        check_fn(cleaned_data.get(field_template.format(delivery_date)))
                    )
                    is False
                    for field_template, check_fn in fields_not_null_check
                ]
            ):
                # Error-highlight all fields on that date.
                for field_template, _check_fn in fields_not_null_check:
                    self.add_error(
                        field_template.format(delivery_date),
                        forms.ValidationError(
                            _("Empty order is not allowed."), code="empty_delivery_date"
                        ),
                    )

        return cleaned_data


class OrderStatusChangeForm(forms.ModelForm):
    class Meta:
        model = OrderStatusChange
        fields = ["order", "status_from", "status_to", "reason"]
        widgets = {
            "status_to": forms.Select(
                attrs={"class": "ui status_to dropdown disabled"}
            ),
            "reason": forms.Textarea(attrs={"rows": 2}),
        }

    def clean(self):
        cleaned_data = super().clean()
        status_to = cleaned_data.get("status_to")
        reason = cleaned_data.get("reason")

        if status_to == "N" and not reason:  # No Charge without reason
            self.add_error(
                "reason",
                forms.ValidationError(
                    _("A reason is required for No Charge order."),
                    code="no_charge_reason_required",
                ),
            )
        return cleaned_data



from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _

from souschef.order.views import (
    CancelOrder,
    CreateDeleteOrderClientBill,
    CreateOrder,
    CreateOrdersBatch,
    DeleteOrder,
    OrderDetail,
    OrderList,
    UpdateOrder,
    UpdateOrderStatus,
)

app_name = "order"

urlpatterns = [
    url(_(r"^list/$"), OrderList.as_view(), name="list"),
    url(_(r"^view/(?P<pk>\d+)/$"), OrderDetail.as_view(), name="view"),
    url(_(r"^create/$"), CreateOrder.as_view(), name="create"),
    # Multiple orders as once
    url(_(r"^create/batch$"), CreateOrdersBatch.as_view(), name="create_batch"),
    url(_(r"^update/(?P<pk>\d+)/$"), UpdateOrder.as_view(), name="update"),
    url(
        _(r"^update/(?P<pk>\d+)/status$"),
        UpdateOrderStatus.as_view(),
        name="update_status",
    ),
    url(
        _(r"^update/(?P<pk>\d+)/client_bill$"),
        CreateDeleteOrderClientBill.as_view(),
        name="update_client_bill",
    ),
    url(_(r"^cancel/(?P<pk>\d+)/$"), CancelOrder.as_view(), name="cancel"),
    url(_(r"^delete/(?P<pk>\d+)/$"), DeleteOrder.as_view(), name="delete"),
]



# Generated by Django 1.9.5 on 2016-06-23 01:15

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0004_fix004a"),
    ]

    operations = [
        migrations.AlterField(
            model_name="client",
            name="route",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="member.Route",
                verbose_name="route",
            ),
        ),
    ]



from __future__ import annotations

import datetime
import json

from annoying.fields import JSONField
from django.db import models
from django.db.models import Q
from django.db.models.functions import Extract
from django.forms import ValidationError
from django.utils import timezone
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _
from django_filters import (
    CharFilter,
    ChoiceFilter,
    FilterSet,
    MultipleChoiceFilter,
)

from souschef.meal.models import (
    COMPONENT_GROUP_CHOICES,
    COMPONENT_GROUP_CHOICES_SIDES,
)
from souschef.member.formsfield import CAPhoneNumberExtField
from souschef.note.models import Note

HOME = "Home phone"
CELL = "Cell phone"
WORK = "Work phone"
EMAIL = "Email"

GENDER_CHOICES = (
    ("F", _("Female")),
    ("M", _("Male")),
    ("O", _("Other")),
)

CONTACT_TYPE_CHOICES = (
    (HOME, HOME),
    (CELL, CELL),
    (WORK, WORK),
    (EMAIL, EMAIL),
)

RATE_TYPE = (
    ("default", _("Default")),
    ("low income", _("Low income")),
    ("solidary", _("Solidary")),
)

RATE_TYPE_LOW_INCOME = RATE_TYPE[1][0]
RATE_TYPE_SOLIDARY = RATE_TYPE[2][0]

PAYMENT_TYPE = (
    (" ", _("----")),
    ("3rd", _("3rd Party")),
    ("cash", _("Cash")),
    ("cheque", _("Cheque")),
    ("credit", _("Credit card")),
    ("eft", _("EFT")),
    ("etransfert", _("e-Transfer")),
)

DELIVERY_TYPE = (
    ("O", _("Ongoing")),
    ("E", _("Episodic")),
)

OPTION_GROUP_CHOICES = (
    ("main dish size", _("Main dish size")),
    ("dish", _("Dish")),
    ("preparation", _("Preparation")),
    ("other order item", _("Other order item")),
)

OPTION_GROUP_CHOICES_PREPARATION = OPTION_GROUP_CHOICES[2][0]

DAYS_OF_WEEK = (
    ("monday", _("Monday")),
    ("tuesday", _("Tuesday")),
    ("wednesday", _("Wednesday")),
    ("thursday", _("Thursday")),
    ("friday", _("Friday")),
    ("saturday", _("Saturday")),
    ("sunday", _("Sunday")),
)

ROUTE_VEHICLES = (
    # Vehicles should be supported by mapbox.
    ("cycling", _("Cycling")),
    ("walking", _("Walking")),
    ("driving", _("Driving")),
)

DEFAULT_VEHICLE = ROUTE_VEHICLES[0][0]


class Member(models.Model):
    class Meta:
        verbose_name_plural = _("members")

    mid = models.IntegerField(blank=True, null=True)

    rid = models.IntegerField(blank=True, null=True)

    # Member information
    firstname = models.CharField(max_length=50, verbose_name=_("First name"))

    lastname = models.CharField(max_length=50, verbose_name=_("Last name"))

    address = models.OneToOneField(
        "member.Address",
        verbose_name=_("address"),
        null=True,
        on_delete=models.SET_NULL,
    )

    work_information = models.TextField(
        verbose_name=_("Work information"),
        blank=True,
        null=True,
    )

    created_at = models.DateTimeField(
        auto_now_add=True,
    )

    updated_at = models.DateTimeField(
        auto_now=True,
    )

    def __str__(self):
        return f"{self.firstname} {self.lastname}"

    @property
    def home_phone(self):
        try:
            val_orig = next(
                mc.value for mc in self.member_contact.all() if mc.type == HOME
            )
            f = CAPhoneNumberExtField()
            val_clean = f.clean(val_orig)
            if val_orig != val_clean:
                self.member_contact.filter(type=HOME).first().value = val_clean
                self.add_contact_information(HOME, val_clean, True)
                return val_clean
            return val_orig
        except ValidationError:
            return val_orig
        except Exception:
            return ""

    @property
    def cell_phone(self):
        try:
            val_orig = next(
                mc.value for mc in self.member_contact.all() if mc.type == CELL
            )
            f = CAPhoneNumberExtField()
            val_clean = f.clean(val_orig)
            if val_orig != val_clean:
                self.member_contact.filter(type=CELL).first().value = val_clean
                self.add_contact_information(CELL, val_clean, True)
                return val_clean
            return val_orig
        except ValidationError:
            return val_orig
        except Exception:
            return ""

    @property
    def work_phone(self):
        try:
            val_orig = next(
                mc.value for mc in self.member_contact.all() if mc.type == WORK
            )
            f = CAPhoneNumberExtField()
            val_clean = f.clean(val_orig)
            if val_orig != val_clean:
                self.member_contact.filter(type=WORK).first().value = val_clean
                self.add_contact_information(WORK, val_clean, True)
                return val_clean
            return val_orig
        except ValidationError:
            return val_orig
        except Exception:
            return ""

    @property
    def email(self):
        try:
            return next(
                mc.value for mc in self.member_contact.all() if mc.type == EMAIL
            )
        except Exception:
            return ""

    def add_contact_information(self, type, value, force_update=False):
        """
        Attach a new contact information to the member instance.
        If a contact information of the given type already exists, it should be
        updated. Otherwise, it should create a new one.
        """
        created = False
        if value is None:
            value = ""

        if force_update or value != "":
            contact, created = Contact.objects.update_or_create(
                member=self, type=type, defaults={"value": value, "member": self}
            )
        return created


class Address(models.Model):
    class Meta:
        verbose_name_plural = _("addresses")
        ordering = ["number", "street", "apartment"]

    # Member address information
    number = models.PositiveIntegerField(
        verbose_name=_("street number"),
        blank=True,
        null=True,
    )

    street = models.CharField(max_length=100, verbose_name=_("street"))

    # Can look like B02 so can't be an IntegerField
    apartment = models.CharField(
        max_length=10,
        verbose_name=_("apartment"),
        blank=True,
        null=True,
    )

    floor = models.IntegerField(
        verbose_name=_("floor"),
        blank=True,
        null=True,
    )

    city = models.CharField(max_length=50, verbose_name=_("city"))

    # Montreal postal code look like H3E 1C2
    postal_code = models.CharField(max_length=7, verbose_name=_("postal code"))

    longitude = models.DecimalField(
        max_digits=9,
        decimal_places=6,
        blank=True,
        null=True,
    )

    latitude = models.DecimalField(
        max_digits=9,
        decimal_places=6,
        blank=True,
        null=True,
    )

    distance = models.DecimalField(
        max_digits=9,
        decimal_places=6,
        blank=True,
        null=True,
    )

    def __str__(self):
        first_line = []
        first_line.append(str(self.street))
        if self.apartment:
            first_line.append(f"Apt.{self.apartment}")
        if self.floor:
            first_line.append(f"{self.floor} Floor")

        first_line = " ".join(first_line)
        second_line = f"{self.city}, {self.postal_code}"

        return f"{first_line}, {second_line}"


class Contact(models.Model):
    class Meta:
        verbose_name_plural = _("contacts")

    # Member contact information
    type = models.CharField(
        max_length=100, choices=CONTACT_TYPE_CHOICES, verbose_name=_("contact type")
    )

    value = models.CharField(max_length=50, verbose_name=_("value"))

    member = models.ForeignKey(
        "member.Member",
        verbose_name=_("member"),
        related_name="member_contact",
        on_delete=models.CASCADE,
    )

    def display_value(self):
        if self.type in (HOME, WORK, CELL):
            try:
                f = CAPhoneNumberExtField()
                return f.clean(self.value)
            except ValidationError:
                return self.value
            except Exception:
                return ""
        return self.value

    def __str__(self):
        return f"{self.member.firstname} {self.member.lastname}"


class Route(models.Model):
    class Meta:
        verbose_name_plural = _("Routes")
        ordering = ["name"]

    name = models.CharField(max_length=50, verbose_name=_("name"))

    description = models.TextField(
        verbose_name=_("description"),
        blank=True,
        null=True,
    )

    vehicle = models.CharField(
        max_length=20,
        choices=ROUTE_VEHICLES,
        verbose_name=_("vehicle"),
        default=DEFAULT_VEHICLE,
    )

    client_id_sequence = JSONField(
        verbose_name=_("IDs of clients on this route (as a JSON list)"), default=[]
    )

    def __str__(self):
        return self.name


class DeliveryHistory(models.Model):
    class Meta:
        verbose_name = _("Delivery History")
        verbose_name_plural = _("Delivery Histories")
        ordering = ["route", "-date"]
        unique_together = ("route", "date")

    route = models.ForeignKey(
        "member.Route",
        on_delete=models.CASCADE,
        verbose_name=_("route"),
        related_name="delivery_histories",
    )
    date = models.DateField(verbose_name=_("date of the delivery"))
    vehicle = models.CharField(
        max_length=20, choices=ROUTE_VEHICLES, verbose_name=_("vehicle")
    )
    client_id_sequence = JSONField(
        verbose_name=_("IDs of clients on this route (as a JSON list)"), default=[]
    )
    comments = models.TextField(
        verbose_name=_("comments"),
        blank=True,
        null=True,
    )

    def __str__(self):
        return f"DeliveryHistory: Route {self.route.name} delivered on {self.date}"


class ClientManager(models.Manager):
    def get_birthday_boys_and_girls(self):
        today = datetime.datetime.now()

        clients = self.filter(
            birthdate__year__lte=today.year,
            birthdate__month=today.month,
            birthdate__day__gte=today.day,
            birthdate__day__lte=today.day + 7,
        ).order_by(Extract("birthdate", "day"))

        today = datetime.date.today()
        for client in clients:
            client.age_to_celebrate = today.year - client.birthdate.year

        return clients


class ActiveClientManager(ClientManager):
    def get_queryset(self):
        return super().get_queryset().filter(status=Client.ACTIVE)


class OngoingClientManager(ClientManager):
    def get_queryset(self):
        return super().get_queryset().filter(status=Client.ACTIVE, delivery_type="O")


class PendingClientManager(ClientManager):
    def get_queryset(self):
        return super().get_queryset().filter(status=Client.PENDING)


class ContactClientManager(ClientManager):
    def get_queryset(self):
        return (
            super()
            .get_queryset()
            .filter(
                Q(status=Client.ACTIVE)
                | Q(status=Client.STOPCONTACT)
                | Q(status=Client.PAUSED)
                | Q(status=Client.PENDING)
            )
        )


class BirthdayContactClientManager(ClientManager):
    def get_queryset(self):
        return (
            super()
            .get_queryset()
            .filter(
                Q(status=Client.ACTIVE)
                | Q(status=Client.PAUSED)
                | Q(status=Client.PENDING)
            )
        )


def get_ongoing_clients_at_date(
    the_date: datetime.date, today: datetime.date | None = None
) -> list["Client"]:  # noqa: UP037
    today = today or datetime.date.today()

    # This way of doing things will not work with a very large client database.
    # Hard-to-write optimized SQL queries would be required.
    return [
        client
        for client in Client.objects.filter(delivery_type="O")
        if client.get_status_planned_at_date(the_date, today) == Client.ACTIVE
    ]


class Client(models.Model):
    # Characters are used to keep a backward-compatibility
    # with the previous system.
    PENDING = "D"
    ACTIVE = "A"
    PAUSED = "S"
    STOPNOCONTACT = "N"
    STOPCONTACT = "C"
    DECEASED = "I"

    ONGOING_DELIVERY = "O"
    EPISODIC_DELIVERY = "E"

    CLIENT_STATUS = (
        (PENDING, _("Pending")),
        (ACTIVE, _("Active")),
        (PAUSED, _("Paused")),
        (STOPNOCONTACT, _("Stop: no contact")),
        (STOPCONTACT, _("Stop: contact")),
        (DECEASED, _("Deceased")),
    )

    LANGUAGES = (
        ("en", _("English")),
        ("fr", _("French")),
        ("al", _("Allophone")),
    )

    class Meta:
        verbose_name_plural = _("clients")
        ordering = ["-member__created_at"]

    billing_member = models.ForeignKey(
        "member.Member",
        related_name="+",
        verbose_name=_("billing member"),
        # A client must have a billing member
        on_delete=models.PROTECT,
    )

    billing_payment_type = models.CharField(
        verbose_name=_("Payment Type"),
        max_length=10,
        null=True,
        blank=True,
        choices=PAYMENT_TYPE,
    )

    rate_type = models.CharField(
        verbose_name=_("rate type"), max_length=10, choices=RATE_TYPE, default="default"
    )

    member = models.ForeignKey(
        "member.Member", verbose_name=_("member"), on_delete=models.CASCADE
    )

    status = models.CharField(max_length=1, choices=CLIENT_STATUS, default=PENDING)

    language = models.CharField(max_length=2, choices=LANGUAGES, default="fr")

    alert = models.TextField(
        verbose_name=_("alert client"),
        blank=True,
        null=True,
    )

    delivery_type = models.CharField(max_length=1, choices=DELIVERY_TYPE, default="O")

    gender = models.CharField(
        max_length=1,
        default="U",
        blank=True,
        null="True",
        choices=GENDER_CHOICES,
    )

    birthdate = models.DateField(
        auto_now=False, auto_now_add=False, default=timezone.now, blank=True, null=True
    )

    route = models.ForeignKey(
        "member.Route",
        on_delete=models.SET_NULL,
        verbose_name=_("route"),
        blank=True,
        null=True,
    )

    meal_default_week = JSONField(blank=True, null=True)

    delivery_note = models.TextField(
        verbose_name=_("Delivery Note"), blank=True, null=True
    )

    ingredients_to_avoid = models.ManyToManyField(
        "meal.Ingredient", through="Client_avoid_ingredient"
    )

    components_to_avoid = models.ManyToManyField(
        "meal.Component", through="Client_avoid_component"
    )

    options = models.ManyToManyField("member.option", through="Client_option")

    restrictions = models.ManyToManyField("meal.Restricted_item", through="Restriction")

    def __str__(self):
        return f"{self.member.firstname} {self.member.lastname}"

    objects = ClientManager()

    active = ActiveClientManager()
    pending = PendingClientManager()
    ongoing = OngoingClientManager()
    contact = ContactClientManager()
    birthday_contact = BirthdayContactClientManager()

    @property
    def is_geolocalized(self):
        """
        Returns if the client's address is properly geolocalized.
        """
        if (
            self.member.address.latitude is None
            or self.member.address.longitude is None
        ):
            return False
        return True

    @property
    def age(self):
        """
        Returns integer specifying person's age in years on the current date.

        To compare the Month and Day they need to be in the same year. Since
        either the birthday can be in a leap year or today can be Feb 29 of a
        leap year, we need to make sure to compare the days, in a leap year,
        therefore we are comparing in the year 2000, which was a leap year.

        >>> from datetime import date
        >>> p = Client(birthdate=date(1950, 4, 19)
        >>> p.age()
        66
        """
        today = datetime.date.today()
        if today < self.birthdate:
            age = 0
        elif datetime.date(
            2000, self.birthdate.month, self.birthdate.day
        ) <= datetime.date(2000, today.month, today.day):
            age = today.year - self.birthdate.year
        else:
            age = today.year - self.birthdate.year - 1
        return age

    @classmethod
    def get_verbose_status(cls, status: str) -> str:
        return dict(cls.CLIENT_STATUS).get(status, _("Unknown"))

    @property
    def status_verbose(self):
        return self.get_verbose_status(self.status)

    @property
    def delivery_type_verbose(self):
        return dict(DELIVERY_TYPE).get(self.delivery_type, _("Unknown"))

    @property
    def orders(self):
        """
        Returns orders associated to this client
        """
        return self.client_order.all()

    def number_of_deliveries_in_month(self, year, month):
        orders = self.client_order.filter(
            status="D", delivery_date__year=year, delivery_date__month=month
        )
        return len(orders)

    @property
    def upcoming_orders(self):
        """
        Returns upcoming orders associated to this client, regardless
        of order status.
        """
        return self.client_order.filter(
            delivery_date__gte=datetime.date.today(),
        ).all()

    @property
    def food_preparation(self):
        """
        Returns specific food preparation associated to this client
        """
        return self.options.filter(option_group="preparation")

    @property
    def notes(self):
        """
        Returns notes associated to this client
        """
        return self.client_notes.all()

    @property
    def simple_meals_schedule(self):
        """
        Returns a list of days, corresponding to the client's delivery
        days.
        """
        for co in self.client_option_set.all():
            if co.option.name == "meals_schedule":
                try:
                    return json.loads(co.value)
                except (ValueError, TypeError):  # JSON error
                    continue
        return None

    @property
    def meals_default(self):
        """
        Returns a list of tuple ((weekday, meal default), ...) that
        represents what the client wants on particular days.

        The "meal default" always contains all available components.
        If not set, it will be None.

        It is possible to have zero value, representing that the client
        has said no to a component on a particular day.
        """
        defaults = []
        for day, _str in DAYS_OF_WEEK:
            current = {}
            numeric_fields = []
            for component, _label in COMPONENT_GROUP_CHOICES:
                if component is COMPONENT_GROUP_CHOICES_SIDES:
                    continue  # skip "Sides"
                item = self.meal_default_week.get(component + "_" + day + "_quantity")
                current[component] = item
                numeric_fields.append(item)

            size = self.meal_default_week.get("size_" + day)
            current["size"] = size

            defaults.append((day, current))

        return defaults

    @property
    def meals_schedule(self):
        """
        Filters `self.meals_default` based on `self.simple_meals_schedule`.
        Non-scheduled days are excluded from the result tuple.

        Intended to be called only for Ongoing clients. For episodic clients
        or if `simple_meals_schedule` is not set, it returns empty tuple.
        """
        defaults = self.meals_default
        prefs = []
        simple_meals_schedule = self.simple_meals_schedule

        if (
            self.delivery_type == self.EPISODIC_DELIVERY
            or simple_meals_schedule is None
        ):
            return ()
        else:
            for day, meal_schedule in defaults:
                if day in simple_meals_schedule:
                    prefs.append((day, meal_schedule))
            return prefs

    def set_simple_meals_schedule(self, schedule):
        """
        Set the delivery days for the client.
        @param schedule
            A python list of days.
        """
        meal_schedule_option, _ = Option.objects.get_or_create(name="meals_schedule")
        client_option, _ = Client_option.objects.update_or_create(
            client=self,
            option=meal_schedule_option,
            defaults={"value": json.dumps(schedule)},
        )

    def get_status_planned_at_date(
        self, the_date: datetime.date, today: datetime.date | None = None
    ):
        """
        Get the planned `status` the client will have at date `the_date`.

        Applies the latest scheduled `ClientScheduledStatus` and return the resulting
        state.

        `today` can be overriden for unit tests.
        """
        today = today or datetime.date.today()

        planned_status = self.status

        status_changes = (
            self.scheduled_statuses.filter(
                change_date__gte=today,
                change_date__lte=the_date,
                change_state=ClientScheduledStatus.END,
                operation_status=ClientScheduledStatus.TOBEPROCESSED,
            )
            .order_by("-change_date")
            .all()
        )

        if status_changes:
            planned_status = status_changes[0].status_to

        return planned_status


class ClientScheduledStatus(models.Model):
    START = "START"
    END = "END"

    CHANGE_STATUS = (
        (START, _("Start")),
        (END, _("End")),
    )

    TOBEPROCESSED = "NEW"
    PROCESSED = "PRO"
    ERROR = "ERR"

    OPERATION_STATUS = (
        (TOBEPROCESSED, _("To be processed")),
        (PROCESSED, _("Processed")),
        (ERROR, _("Error")),
    )

    client = models.ForeignKey(
        Client, on_delete=models.CASCADE, related_name="scheduled_statuses"
    )

    pair = models.OneToOneField(
        "self",
        verbose_name=_("Client Scheduled Status Pair"),
        related_name="my_pair",
        on_delete=models.CASCADE,
        blank=True,
        null=True,
    )

    status_from = models.CharField(max_length=1, choices=Client.CLIENT_STATUS)

    status_to = models.CharField(max_length=1, choices=Client.CLIENT_STATUS)

    reason = models.CharField(max_length=200, blank=True, default="")

    change_date = models.DateField(
        auto_now=False,
        auto_now_add=False,
        default=timezone.now,
    )

    change_state = models.CharField(max_length=5, choices=CHANGE_STATUS, default=START)

    operation_status = models.CharField(
        max_length=3, choices=OPERATION_STATUS, default=TOBEPROCESSED
    )

    class Meta:
        ordering = ["change_date"]

    def __str__(self):
        return (
            f"Update {self.client.member} status: from "
            f"{self.get_status_from_display()} to {self.get_status_to_display()}, "
            f"on {self.change_date}"
        )

    @property
    def get_pair(self):
        """
        Returns the pair relationship. If the pair relationship does
        not exists, None value will be returned.
        :return:  ClientScheduledStatus
        """
        try:
            return self.pair or self.my_pair
        except ClientScheduledStatus.DoesNotExist:
            return None

    def process(self):
        """Process a scheduled change if valid."""
        if self.is_valid():
            # Update the client status
            self.client.status = self.status_to
            self.client.save()
            # Update the instance status
            self.operation_status = self.PROCESSED
            self.save()
            # Add note to client
            self.add_note_to_client()
            return True
        else:
            self.operation_status = self.ERROR
            self.save()
            return False

    def is_valid(self):
        """Returns True if a scheduled change must be applied."""
        return (
            self.client.status == self.status_from
            and self.operation_status == self.TOBEPROCESSED
        )

    def add_note_to_client(self, author=None):
        note = Note(
            note=self,
            author=author,
            client=self.client,
        )
        note.save()

    @property
    def needs_attention(self):
        """
        Return True if the status is ERROR or the scheduled date has passed.
        """
        return (self.operation_status == self.ERROR) or (
            self.change_date <= timezone.datetime.date(timezone.datetime.today())
        )


class ClientScheduledStatusFilter(FilterSet):
    class Meta:
        model = ClientScheduledStatus
        fields = ["operation_status"]


class ClientFilter(FilterSet):
    name = CharFilter(method="filter_search", label=_("Search by name"))

    status = MultipleChoiceFilter(choices=Client.CLIENT_STATUS)

    delivery_type = ChoiceFilter(choices=(("", ""),) + DELIVERY_TYPE)

    class Meta:
        model = Client
        fields = ["route", "status", "delivery_type"]

    def filter_search(self, queryset, field_name, value):
        if not value:
            return queryset

        name_contains = Q()
        names = value.split(" ")

        for name in names:
            firstname_contains = Q(member__firstname__icontains=name)

            lastname_contains = Q(member__lastname__icontains=name)

            name_contains |= firstname_contains | lastname_contains

        return queryset.filter(name_contains)


class Relationship(models.Model):
    REFERENT = "referent"
    EMERGENCY = "emergency"
    TYPE_CHOICES = (
        (REFERENT, _("Referent")),
        (EMERGENCY, _("Emergency")),
    )
    TYPE_CHOICES_DICT = dict(TYPE_CHOICES)

    member = models.ForeignKey("member.Member", on_delete=models.CASCADE)
    client = models.ForeignKey("member.Client", on_delete=models.CASCADE)
    nature = models.CharField(max_length=100)
    # Relationship.type is linked with a forms.MultipleChoiceField
    type = JSONField(default=[], blank=True)
    extra_fields = JSONField(default={}, blank=True)
    remark = models.TextField(blank=True)

    def __str__(self):
        return (
            f"{self.member.firstname} {self.member.lastname} is "
            f"{self.get_type_display()} member of {self.client.member.firstname} "
            f"{self.client.member.lastname}"
        )

    def get_type_display(self):
        return "+".join(
            map(
                force_text,
                list(map(lambda t: self.TYPE_CHOICES_DICT[t], self.type))
                or ([_("Unknown type")]),
            )
        )

    class Meta:
        verbose_name_plural = _("relationships")
        unique_together = ("client", "member")

    def is_referent(self):
        return self.REFERENT in self.type

    def is_emergency(self):
        return self.EMERGENCY in self.type


class Option(models.Model):
    class Meta:
        verbose_name_plural = _("options")

    # Information about options added to the meal
    name = models.CharField(max_length=50, verbose_name=_("name"))

    description = models.TextField(
        verbose_name=_("description"),
        blank=True,
        null=True,
    )

    option_group = models.CharField(
        max_length=100, choices=OPTION_GROUP_CHOICES, verbose_name=_("option group")
    )

    def __str__(self):
        return self.name


class Client_option(models.Model):
    client = models.ForeignKey(
        "member.Client", verbose_name=_("client"), on_delete=models.CASCADE
    )

    option = models.ForeignKey(
        "member.option", verbose_name=_("option"), on_delete=models.CASCADE
    )

    value = models.CharField(max_length=255, null=True, verbose_name=_("value"))
    #  value contents depends on option_group of option occurence pointed to:
    #    if option_group = main_dish_size : 'Regular' or 'Large'
    #    if option_group = dish : qty Monday to Sunday ex. '1110120'
    #    if option_group = preparation : Null
    #    if option_group = other_order_item : No occurrence of Client_option

    def __str__(self):
        return (
            f"{self.client.member.firstname} {self.client.member.lastname} <has> "
            f"{self.option.name}"
        )


class Restriction(models.Model):
    client = models.ForeignKey(
        "member.Client",
        verbose_name=_("client"),
        related_name="+",
        on_delete=models.CASCADE,
    )

    restricted_item = models.ForeignKey(
        "meal.Restricted_item",
        verbose_name=_("restricted item"),
        related_name="+",
        on_delete=models.CASCADE,
    )

    def __str__(self):
        return (
            f"{self.client.member.firstname} {self.client.member.lastname} "
            f"<restricts> {self.restricted_item.name}"
        )


class Client_avoid_ingredient(models.Model):
    client = models.ForeignKey(
        "member.Client",
        verbose_name=_("client"),
        related_name="+",
        on_delete=models.CASCADE,
    )

    ingredient = models.ForeignKey(
        "meal.Ingredient",
        verbose_name=_("ingredient"),
        related_name="+",
        on_delete=models.CASCADE,
    )

    def __str__(self):
        return (
            f"{self.client.member.firstname} {self.client.member.lastname} "
            f"<has> {self.ingredient.name}"
        )


class Client_avoid_component(models.Model):
    client = models.ForeignKey(
        "member.Client",
        verbose_name=_("client"),
        related_name="+",
        on_delete=models.CASCADE,
    )

    component = models.ForeignKey(
        "meal.Component",
        verbose_name=_("component"),
        related_name="+",
        on_delete=models.CASCADE,
    )

    def __str__(self):
        return (
            f"{self.client.member.firstname} {self.client.member.lastname} "
            f"<has> {self.component.name}"
        )



import csv

from django.core.management.base import BaseCommand

from souschef.member.models import (
    Client,
    Client_option,
    Member,
    Option,
)


class Command(BaseCommand):
    """
    **Warning!**
    This command has been modified without keeping trace of why a long time
    ago and the tests were outdated. Please try to avoid using this function
    since we will try to deprecate it in the future.
    """

    help = "Data: import clients relationships from given csv file."

    ROW_MID = 0
    ROW_MON = 1
    ROW_TUE = 2
    ROW_WED = 3
    ROW_THU = 4
    ROW_FRI = 5
    ROW_SAT = 6
    ROW_FOOD_PREP_PUREE = 7
    ROW_FOOD_PREP_CUT = 8
    ROW_MLABEL = 9
    ROW_ING_BEEF = 10

    def add_arguments(self, parser):
        parser.add_argument(
            "--file",
            default=False,
            help="Import mock data instead of actual data",
        )

    def handle(self, *args, **options):
        file = "mock_meals.csv" if options["file"] else "clients_meals.csv"

        food_prep_puree = Option.objects.get(name="Puree all")
        food_prep_cut = Option.objects.get(name="Cut up meat")

        with open(file) as f:
            reader = csv.reader(f, delimiter=";")
            for row in reader:
                try:
                    member = Member.objects.get(mid=row[self.ROW_MID])
                    client = Client.objects.get(member=member)
                    days = [
                        self.ROW_MON,
                        self.ROW_TUE,
                        self.ROW_WED,
                        self.ROW_THU,
                        self.ROW_FRI,
                        self.ROW_SAT,
                    ]
                    meals_schedule = []
                    prefs = {}

                    for day in days:
                        if row[day] != "":
                            meals_schedule.append(row[day])
                            delivery_day = day - 1
                            # Hack: there is no column Thursday in the csv file
                            if delivery_day > 3:
                                delivery_day -= 1

                            prefs["size_" + row[day]] = row[
                                11 + (delivery_day * 10) + 1
                            ]
                            prefs["main_dish_" + row[day] + "_quantity"] = int(
                                row[11 + (delivery_day * 10) + 0]
                            )
                            prefs["dessert_" + row[day] + "_quantity"] = int(
                                row[11 + (delivery_day * 10) + 5]
                            )
                            prefs["fruit_salad_" + row[day] + "_quantity"] = int(
                                row[11 + (delivery_day * 10) + 2]
                            )
                            prefs["green_salad_" + row[day] + "_quantity"] = int(
                                row[11 + (delivery_day * 10) + 3]
                            )
                            prefs["pudding_" + row[day] + "_quantity"] = int(
                                row[11 + (delivery_day * 10) + 6]
                            )
                            prefs["diabetic_" + row[day] + "_quantity"] = int(
                                row[11 + (delivery_day * 10) + 4]
                            )

                    client.set_simple_meals_schedule(meals_schedule)
                    client.meal_default_week = prefs
                    client.save()

                    # Food preparation
                    if row[self.ROW_FOOD_PREP_CUT] == "1":
                        Client_option.objects.create(
                            client=client, option=food_prep_cut
                        )
                    if row[self.ROW_FOOD_PREP_PUREE] == "1":
                        Client_option.objects.create(
                            client=client, option=food_prep_puree
                        )

                except Member.DoesNotExist:
                    self.stdout.write(self.style.WARNING("Non existing member"))
                except Client.DoesNotExist:
                    self.stdout.write(self.style.WARNING("Non existing client"))



from django.core.management.commands import makemessages


class Command(makemessages.Command):
    """
    Support argument --add-location of GNU gettext utilities.

    A feature request has been submitted to Django at the time of writing
    this module (Django 1.11rc1). If a future Django version supports this
    argument, we can then safely remove this module.

    Ref: https://groups.google.com/forum/#!topic/django-developers/IkrjfBDA7iE
    """

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            "--add-location",
            action="store",
            default="full",
            type=str,
            choices=["full", "file", "never"],
            help="If full, generates the location lines with both file name "
            "and line number. If file, the line number part is omitted. "
            "If never, completely suppresses the lines "
            "(same as --no-location).",
            dest="add_location",
        )

    def handle(self, *args, **options):
        add_location = options.pop("add_location")
        arg_str = f"--add-location={add_location}"
        self.msgmerge_options = makemessages.Command.msgmerge_options[:] + [arg_str]
        self.msguniq_options = makemessages.Command.msguniq_options[:] + [arg_str]
        self.msgattrib_options = makemessages.Command.msgattrib_options[:] + [arg_str]
        self.xgettext_options = makemessages.Command.xgettext_options[:] + [arg_str]

        super().handle(*args, **options)



# Generated by Django 1.10 on 2016-08-19 20:37

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("note", "0002_auto_20160818_2104"),
    ]

    operations = [
        migrations.AlterField(
            model_name="note",
            name="date",
            field=models.DateTimeField(
                default=django.utils.timezone.now, verbose_name="Date"
            ),
        ),
    ]



from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _

from souschef.billing.views import (
    BillingAdd,
    BillingCreate,
    BillingDelete,
    BillingList,
    BillingOrdersView,
    BillingSummaryView,
)

app_name = "billing"

urlpatterns = [
    url(_(r"^list/$"), BillingList.as_view(), name="list"),
    url(_(r"^create/$"), BillingCreate.as_view(), name="create"),
    url(_(r"^add/$"), BillingAdd.as_view(), name="add"),
    url(_(r"^view/(?P<pk>\d+)/$"), BillingSummaryView.as_view(), name="view"),
    url(
        _(r"^view/(?P<pk>\d+)/orders/$"),
        BillingOrdersView.as_view(),
        name="view_orders",
    ),
    url(_(r"^delete/(?P<pk>\d+)/$"), BillingDelete.as_view(), name="delete"),
]



"""sous-chef URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/1.9/topics/http/urls/

Examples:

Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  url(r'^$', views.home, name='home')

Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  url(r'^$', Home.as_view(), name='home')

Including another URLconf
    1. Import the include() function: from django.conf.urls import url, include
    2. Add a URL to urlpatterns:  url(r'^blog/', include('blog.urls'))

"""
from django.conf import settings
from django.conf.urls import (
    include,
    url,
)
from django.conf.urls.i18n import i18n_patterns
from django.conf.urls.static import static
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from souschef.page.views import HomeView

urlpatterns = i18n_patterns(
    url(_(r"^admin/"), admin.site.urls),
    url(_(r"^meal/"), include("souschef.meal.urls", namespace="meal")),
    url(_(r"^member/"), include("souschef.member.urls", namespace="member")),
    url(
        _(r"^notification/"),
        include("souschef.notification.urls", namespace="notification"),
    ),
    url(_(r"^order/"), include("souschef.order.urls", namespace="order")),
    url(r"^p/", include("souschef.page.urls", namespace="page")),
    url(r"^$", HomeView.as_view(), name="home"),
    url(_(r"^delivery/"), include("souschef.delivery.urls", namespace="delivery")),
    url(_(r"^note/"), include("souschef.note.urls", namespace="note")),
    url(_(r"^billing/"), include("souschef.billing.urls", namespace="billing")),
    url(_(r"^avatar/"), include("avatar.urls")),
    prefix_default_language=False,
) + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



from django.core.management.base import BaseCommand

from souschef.order.factories import OrderFactory


class Command(BaseCommand):
    help = "Creates a happy bunch of clients with orders"

    def handle(self, *args, **options):
        orders = OrderFactory.create_batch(500, status="D")
        self.stdout.write(
            self.style.SUCCESS(f"Successfully created client {orders} orders")
        )



# Generated by Django 1.9.8 on 2016-08-17 16:48

import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0011_client_delivery_note"),
    ]

    operations = [
        migrations.CreateModel(
            name="ClientScheduledStatus",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "status_from",
                    models.CharField(
                        choices=[
                            ("D", "Pending"),
                            ("A", "Active"),
                            ("S", "Paused"),
                            ("N", "Stop: no contact"),
                            ("C", "Stop: contact"),
                            ("I", "Deceased"),
                        ],
                        max_length=1,
                    ),
                ),
                (
                    "status_to",
                    models.CharField(
                        choices=[
                            ("D", "Pending"),
                            ("A", "Active"),
                            ("S", "Paused"),
                            ("N", "Stop: no contact"),
                            ("C", "Stop: contact"),
                            ("I", "Deceased"),
                        ],
                        max_length=1,
                    ),
                ),
                ("reason", models.CharField(blank=True, default="", max_length=200)),
                (
                    "change_date",
                    models.DateField(
                        blank=True, default=django.utils.timezone.now, null=True
                    ),
                ),
                (
                    "change_state",
                    models.CharField(
                        choices=[
                            ("ALONE", "Alone"),
                            ("START", "Start"),
                            ("END", "End"),
                        ],
                        default="ALONE",
                        max_length=5,
                    ),
                ),
                (
                    "operation_status",
                    models.CharField(
                        choices=[
                            ("NEW", "To be processed"),
                            ("PRO", "Processed"),
                            ("ERR", "Error"),
                        ],
                        default="NEW",
                        max_length=3,
                    ),
                ),
                (
                    "client",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="member.Client"
                    ),
                ),
                (
                    "linked_scheduled_status",
                    models.ForeignKey(
                        blank=True,
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="member.ClientScheduledStatus",
                    ),
                ),
            ],
        ),
    ]



from django import forms
from django.core.exceptions import ObjectDoesNotExist
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from localflavor.ca.forms import CAPostalCodeField

from souschef.meal.models import (
    COMPONENT_GROUP_CHOICES,
    COMPONENT_GROUP_CHOICES_SIDES,
    Component,
    Ingredient,
    Restricted_item,
)
from souschef.member.formsfield import CAPhoneNumberExtField
from souschef.member.models import (
    DAYS_OF_WEEK,
    DELIVERY_TYPE,
    GENDER_CHOICES,
    PAYMENT_TYPE,
    RATE_TYPE,
    Client,
    ClientScheduledStatus,
    Member,
    Option,
    Relationship,
    Route,
)
from souschef.order.models import SIZE_CHOICES


class ClientBasicInformation(forms.Form):
    firstname = forms.CharField(
        max_length=100,
        label=_("First Name"),
        widget=forms.TextInput(attrs={"placeholder": _("First name")}),
    )

    lastname = forms.CharField(
        max_length=100,
        label=_("Last Name"),
        widget=forms.TextInput(attrs={"placeholder": _("Last name")}),
    )

    gender = forms.ChoiceField(
        choices=GENDER_CHOICES, widget=forms.Select(attrs={"class": "ui dropdown"})
    )

    language = forms.ChoiceField(
        choices=Client.LANGUAGES,
        label=_("Preferred language"),
        widget=forms.Select(attrs={"class": "ui dropdown"}),
    )

    birthdate = forms.DateField(
        label=_("Birthday"),
        widget=forms.TextInput(
            attrs={"class": "ui calendar", "placeholder": _("YYYY-MM-DD")}
        ),
        help_text=_("Format: YYYY-MM-DD"),
    )

    email = forms.EmailField(
        max_length=100,
        label='<i class="email icon"></i>',
        widget=forms.TextInput(attrs={"placeholder": _("Email")}),
        required=False,
    )

    home_phone = CAPhoneNumberExtField(
        label="Home",
        widget=forms.TextInput(attrs={"placeholder": _("Home phone")}),
        required=False,
    )

    cell_phone = CAPhoneNumberExtField(
        label="Cell",
        widget=forms.TextInput(attrs={"placeholder": _("Cellular")}),
        required=False,
    )

    alert = forms.CharField(
        label=_("Alert"),
        required=False,
        widget=forms.Textarea(
            attrs={"rows": 2, "placeholder": _("Your message here ...")}
        ),
    )

    def clean(self):
        if not (
            self.cleaned_data.get("email")
            or self.cleaned_data.get("home_phone")
            or self.cleaned_data.get("cell_phone")
        ):
            msg = _("At least one contact information is required.")
            self.add_error("email", msg)
            self.add_error("home_phone", msg)
            self.add_error("cell_phone", msg)

        return self.cleaned_data


class ClientAddressInformation(forms.Form):
    apartment = forms.CharField(
        label=_("Apt #"),
        widget=forms.TextInput(attrs={"placeholder": _("Apt #"), "class": "apartment"}),
        required=False,
    )

    street = forms.CharField(
        max_length=100,
        label=_("Address Information"),
        widget=forms.TextInput(
            attrs={"placeholder": _("7275 Rue Saint-Urbain"), "class": "street name"}
        ),
    )

    city = forms.CharField(
        max_length=50,
        label=_("City"),
        widget=forms.TextInput(attrs={"placeholder": _("Montreal"), "class": "city"}),
    )

    postal_code = CAPostalCodeField(
        max_length=7,
        label=_("Postal Code"),
        widget=forms.TextInput(
            attrs={"placeholder": _("H2R 2Y5"), "class": "postal code"}
        ),
    )

    latitude = forms.CharField(
        label=_("Latitude"),
        required=False,
        initial=0,
        widget=forms.TextInput(attrs={"class": "latitude"}),
    )

    longitude = forms.CharField(
        label=_("Longitude"),
        required=False,
        initial=0,
        widget=forms.TextInput(attrs={"class": "longitude"}),
    )

    distance = forms.CharField(
        label=_("Distance from Santropol"),
        required=False,
        initial=0,
        widget=forms.TextInput(attrs={"class": "distance"}),
    )

    route = forms.ModelChoiceField(
        label=_("Route"),
        required=True,
        widget=forms.Select(attrs={"class": "ui search dropdown"}),
        queryset=Route.objects.all(),
    )

    delivery_note = forms.CharField(
        label=_("Delivery Note"),
        required=False,
        widget=forms.Textarea(
            attrs={"rows": 2, "placeholder": _("Delivery Note here ...")}
        ),
    )


class ClientRestrictionsInformation(forms.Form):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        for day, _translation in DAYS_OF_WEEK:
            self.fields[f"size_{day}"] = forms.ChoiceField(
                choices=SIZE_CHOICES, widget=forms.Select(), required=False
            )

            for meal, _meal_translation in COMPONENT_GROUP_CHOICES:
                if meal is COMPONENT_GROUP_CHOICES_SIDES:
                    continue  # skip "Sides"
                self.fields[f"{meal}_{day}_quantity"] = forms.IntegerField(
                    required=False, min_value=0, widget=forms.TextInput()
                )

    status = forms.BooleanField(
        label=_("Active"),
        help_text=_("By default, the client meal status is Pending."),
        required=False,
    )

    delivery_type = forms.ChoiceField(
        label=_("Type"),
        choices=DELIVERY_TYPE,
        required=True,
        widget=forms.Select(attrs={"class": "ui dropdown"}),
    )

    meals_schedule = forms.MultipleChoiceField(
        label=_("Schedule"),
        initial="Select days of week",
        choices=DAYS_OF_WEEK,
        widget=forms.SelectMultiple(attrs={"class": "ui dropdown"}),
        required=False,
    )

    restrictions = forms.ModelMultipleChoiceField(
        label=_("Restrictions"),
        queryset=Restricted_item.objects.all(),
        required=False,
        widget=forms.SelectMultiple(attrs={"class": "ui dropdown search"}),
    )

    food_preparation = forms.ModelMultipleChoiceField(
        label=_("Preparation"),
        required=False,
        queryset=Option.objects.filter(option_group="preparation"),
        widget=forms.SelectMultiple(attrs={"class": "ui dropdown"}),
    )

    ingredient_to_avoid = forms.ModelMultipleChoiceField(
        label=_("Ingredient To Avoid"),
        queryset=Ingredient.objects.all(),
        required=False,
        widget=forms.SelectMultiple(attrs={"class": "ui dropdown search"}),
    )

    dish_to_avoid = forms.ModelMultipleChoiceField(
        label=_("Dish(es) To Avoid"),
        queryset=Component.objects.all(),
        required=False,
        widget=forms.SelectMultiple(attrs={"class": "ui dropdown search"}),
    )

    def clean(self):
        """
        The meal defaults are required for the scheduled delivery days:
        at least one of the quantities should be set.
        This only applies to ongoing clients!

        Regardless of meal schedules, when a main dish is set, we should
        enforce the setting of its size.
        """
        super().clean()

        if self.cleaned_data.get("delivery_type") == Client.ONGOING_DELIVERY:
            # Ongoing
            meals_schedule = self.cleaned_data.get("meals_schedule")
            if meals_schedule is None:
                meals_schedule = []
        else:
            # Episodic
            meals_schedule = []

        day_displays = dict(DAYS_OF_WEEK)

        for day in meals_schedule:
            # At least one of the quantities should be set.
            quantity_fieldnames = []
            for meal, _meal_translation in COMPONENT_GROUP_CHOICES:
                if meal is COMPONENT_GROUP_CHOICES_SIDES:
                    continue  # skip "Sides"
                fieldname = f"{meal}_{day}_quantity"
                quantity_fieldnames.append(fieldname)

            total_quantity = sum(
                map(lambda n: self.cleaned_data.get(n) or 0, quantity_fieldnames), 0
            )
            if total_quantity == 0:
                for n in quantity_fieldnames:
                    self.add_error(
                        n,
                        _(
                            "At least one of the quantities should be "
                            "set because %(weekday)s is scheduled for "
                            "delivery."
                        )
                        % {"weekday": day_displays[day]},
                    )

        for day, _day_display in DAYS_OF_WEEK:
            # If the main dish is set, size should also be set.
            main_dish_quantity = self.cleaned_data.get(f"main_dish_{day}_quantity")
            fieldname_size = f"size_{day}"
            if main_dish_quantity and not self.cleaned_data.get(fieldname_size):
                self.add_error(
                    fieldname_size, _("Size is required when the main dish is set.")
                )


class MemberForm(forms.Form):
    member = forms.CharField(
        label=_("Member"),
        widget=forms.TextInput(
            attrs={
                "placeholder": _("Type 3 characters or more to search members"),
                "class": "prompt existing--member",
            }
        ),
        required=False,
    )

    firstname = forms.CharField(
        label=_("First Name"),
        widget=forms.TextInput(
            attrs={"placeholder": _("First Name"), "class": "firstname"}
        ),
        required=False,
    )

    lastname = forms.CharField(
        label=_("Last Name"),
        widget=forms.TextInput(
            attrs={"placeholder": _("Last Name"), "class": "lastname"}
        ),
        required=False,
    )

    email = forms.EmailField(
        label='<i class="email icon"></i>',
        widget=forms.TextInput(attrs={"placeholder": _("Email")}),
        required=False,
    )

    work_phone = CAPhoneNumberExtField(
        label="Work",
        widget=forms.TextInput(attrs={"placeholder": _("Work phone")}),
        required=False,
    )

    cell_phone = CAPhoneNumberExtField(
        label="Cell",
        widget=forms.TextInput(attrs={"placeholder": _("Cellular")}),
        required=False,
    )

    def clean(self):
        cleaned_data = super().clean()

        """If the client pays for himself. """
        if cleaned_data.get("same_as_client") is True:
            return cleaned_data

        member = cleaned_data.get("member")
        firstname = cleaned_data.get("firstname")
        lastname = cleaned_data.get("lastname")

        if not member and (not firstname or not lastname):
            msg = _("This field is required unless you add a new member.")
            self.add_error("member", msg)
            msg = _("This field is required unless you chose an existing member.")
            self.add_error("firstname", msg)
            self.add_error("lastname", msg)

        if member:
            member_id = member.split(" ")[0].replace("[", "").replace("]", "")
            try:
                Member.objects.get(pk=member_id)
            except ObjectDoesNotExist:
                msg = _("Not a valid member, please chose an existing member.")
                self.add_error("member", msg)
        return cleaned_data


class ClientPaymentInformation(MemberForm):
    facturation = forms.ChoiceField(
        label=_("Billing Type"),
        choices=RATE_TYPE,
        widget=forms.Select(attrs={"class": "ui dropdown"}),
    )

    same_as_client = forms.BooleanField(
        label=_("Same As Client"),
        required=False,
        help_text=_(
            "If checked, the personal information "
            "of the client will be used as billing information."
        ),
        widget=forms.CheckboxInput(attrs={}),
    )

    billing_payment_type = forms.ChoiceField(
        label=_("Payment Type"),
        choices=PAYMENT_TYPE,
        widget=forms.Select(attrs={"class": "ui dropdown"}),
        required=False,
    )

    number = forms.IntegerField(label=_("Street Number"), required=False)

    apartment = forms.CharField(
        label=_("Apt #"),
        widget=forms.TextInput(attrs={"placeholder": _("Apt #")}),
        required=False,
    )

    floor = forms.IntegerField(label=_("Floor"), required=False)

    street = forms.CharField(label=_("Street Name"), required=False)

    city = forms.CharField(label=_("City Name"), required=False)

    postal_code = CAPostalCodeField(label=_("Postal Code"), required=False)

    def clean(self):
        cleaned_data = super().clean()

        if cleaned_data.get("same_as_client") is True:
            return cleaned_data

        member = cleaned_data.get("member")
        if member:
            member_id = member.split(" ")[0].replace("[", "").replace("]", "")
            member_obj = Member.objects.get(pk=member_id)
            if not member_obj.address:
                msg = _(
                    "This member has not a valid address, "
                    "please add a valid address to this member, so it can "
                    "be used for the billing."
                )
                self.add_error("member", msg)
        else:
            msg = _("This field is required")
            fields = ["street", "city", "postal_code"]
            for field in fields:
                field_data = cleaned_data.get(field)
                if not field_data:
                    self.add_error(field, msg)
        return cleaned_data


class ClientRelationshipInformation(MemberForm):
    nature = forms.CharField(
        label=_("Nature of Relationship"),
        widget=forms.TextInput(
            attrs={"placeholder": _("Doctor, Friend, Daughter, ...")}
        ),
    )

    type = forms.MultipleChoiceField(
        label=_("Type of Relationship"),
        choices=Relationship.TYPE_CHOICES,
        widget=forms.CheckboxSelectMultiple(
            attrs={
                "onchange": '$(this).closest(".formset-item.relationship")'
                '.find(".souschef-referent-only")'
                '[($(this).closest("ul").find('
                '        "input[type=checkbox][value=%s]:checked"'
                "    ).length > 0) ? "
                ' "show" : "hide"]()' % Relationship.REFERENT,
                "class": "sc-checkbox-select-multiple",
            }
        ),
        initial=[],
        required=False,
    )

    remark = forms.CharField(
        label=_("Remark"), widget=forms.Textarea(attrs={"rows": 4}), required=False
    )

    # Extra fields
    work_information = forms.CharField(
        max_length=200,
        label=_("Work information"),
        widget=forms.TextInput(
            attrs={"placeholder": _("Hotel-Dieu, St-Anne Hospital, ...")}
        ),
        required=False,
    )
    referral_date = forms.DateField(
        label=_("Referral Date"),
        widget=forms.TextInput(
            attrs={"class": "ui calendar", "placeholder": _("YYYY-MM-DD")}
        ),
        help_text=_("Format: YYYY-MM-DD"),
        required=False,
    )
    referral_reason = forms.CharField(
        label=_("Referral Reason"),
        widget=forms.Textarea(attrs={"rows": 4}),
        required=False,
    )

    @property
    def has_referent_relationship(self):
        return Relationship.REFERENT in self.cleaned_data.get("type", [])

    def clean(self):
        member = self.cleaned_data.get("member")
        if member:
            member_id = member.split(" ")[0].replace("[", "").replace("]", "")
            try:
                Member.objects.get(pk=member_id)
            except (ObjectDoesNotExist, ValueError):
                msg = _("Not a valid member, please chose an existing member.")
                self.add_error("member", msg)
        else:
            if not self.cleaned_data.get("firstname"):
                msg = _(
                    "This field is required unless " "you chose an existing member."
                )
                self.add_error("firstname", msg)
            if not self.cleaned_data.get("firstname"):
                msg = _(
                    "This field is required unless " "you chose an existing member."
                )
                self.add_error("lastname", msg)
            if not (
                self.cleaned_data.get("email")
                or self.cleaned_data.get("work_phone")
                or self.cleaned_data.get("cell_phone")
            ):
                msg = _("At least one contact is required.")
                self.add_error("email", msg)
                self.add_error("work_phone", msg)
                self.add_error("cell_phone", msg)
            if self.has_referent_relationship and not self.cleaned_data.get(
                "work_information"
            ):
                msg = _("This field is required for a referent relationship.")
                self.add_error("work_information", msg)

        if self.has_referent_relationship:
            if not self.cleaned_data.get("referral_date"):
                msg = _("This field is required for a referent relationship.")
                self.add_error("referral_date", msg)
            if not self.cleaned_data.get("referral_reason"):
                msg = _("This field is required for a referent relationship.")
                self.add_error("referral_reason", msg)
        return self.cleaned_data


class ClientScheduledStatusForm(forms.ModelForm):
    class Meta:
        model = ClientScheduledStatus
        fields = ["client", "status_from", "status_to", "reason", "change_date"]
        widgets = {
            "status_to": forms.Select(attrs={"class": "ui status_to dropdown"}),
            "reason": forms.Textarea(attrs={"rows": 2}),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["end_date"] = forms.DateField(
            required=False,
            widget=forms.TextInput(
                attrs={"class": "ui calendar", "placeholder": _("YYYY-MM-DD")}
            ),
            help_text=_("Format: YYYY-MM-DD"),
        )

    def save(self, *args, **kwargs):
        """
        Override the default behavior of a ModelForm.
        We may have two ClientScheduledStatus instances to save. The .save()
        method is not expressive enough for what we want to do.
        Thus, we raise an error when trying to call form.save() to remind
        other people of using the custom save method.
        """
        raise NotImplementedError(
            "This method is intentionally bypassed. Please use "
            "save_scheduled_statuses method."
        )

    def save_scheduled_statuses(self, callback_add_message=lambda m: None):
        """
        Create one or two ClientScheduledStatus instances according to
        `self.cleaned_data`.
        """
        if not hasattr(self, "cleaned_data") or self.errors:
            raise ValueError("The form data doesn't validate.")
        data = self.cleaned_data
        today = timezone.datetime.date(timezone.datetime.today())

        # Save and process instance(s)
        c1 = ClientScheduledStatus.objects.create(
            client=data["client"],
            status_from=data["status_from"],
            status_to=data["status_to"],
            reason=data["reason"],
            change_date=data["change_date"],
            change_state=ClientScheduledStatus.END,
            operation_status=ClientScheduledStatus.TOBEPROCESSED,
        )
        if data["change_date"] == today:
            c1.process()
            callback_add_message(_("The client status has been changed."))
        else:
            callback_add_message(_("This status change has been scheduled."))

        if data.get("end_date"):
            c2 = ClientScheduledStatus.objects.create(
                client=data["client"],
                status_from=data["status_to"],
                status_to=data["status_from"],
                reason=data["reason"],
                change_date=data["end_date"],
                change_state=ClientScheduledStatus.END,
                operation_status=ClientScheduledStatus.TOBEPROCESSED,
                pair=c1,
            )
            if data.get("end_date") == today:
                c2.process()
                callback_add_message(_("The client status has been changed."))
            else:
                callback_add_message(
                    _("The end date of this status change " "has been scheduled.")
                )



from django.contrib import admin

from souschef.member.models import (
    Address,
    Client,
    ClientScheduledStatus,
    Contact,
    DeliveryHistory,
    Member,
    Option,
    Relationship,
    Route,
)


class IngredientsToAvoidInline(admin.TabularInline):
    model = Client.ingredients_to_avoid.through


class ComponentsToAvoidInline(admin.TabularInline):
    model = Client.components_to_avoid.through


class RestrictionsInline(admin.TabularInline):
    model = Client.restrictions.through


class OptionsInline(admin.TabularInline):
    model = Client.options.through


class ContactInline(admin.TabularInline):
    model = Contact


class RelationshipInline(admin.TabularInline):
    model = Relationship
    extra = 0


class MemberAdmin(admin.ModelAdmin):
    search_fields = ["lastname", "firstname"]
    list_display = ("full_name", "address", "work_information", "updated_at")
    inlines = [ContactInline]

    def full_name(self, obj):
        return f"{obj.firstname} {obj.lastname}"

    full_name.short_description = "Name"


class ClientAdmin(admin.ModelAdmin):
    search_fields = ["member__lastname", "member__firstname"]
    list_filter = ("status", "route", "delivery_type")
    list_display = ("member", "status", "language", "delivery_type", "gender", "route")
    inlines = [
        RelationshipInline,
        OptionsInline,
        RestrictionsInline,
        ComponentsToAvoidInline,
        IngredientsToAvoidInline,
    ]


class ContactAdmin(admin.ModelAdmin):
    search_fields = ["member__lastname", "member__firstname"]
    list_display = (
        "member",
        "type",
        "value",
    )
    list_filter = ("type",)


class RelationshipAdmin(admin.ModelAdmin):
    search_fields = [
        "member__lastname",
        "member__firstname",
        "client__member__lastname",
        "client__member__firstname",
        "type",
        "nature",
        "extra_fields",
    ]
    list_display = ("__str__", "member", "type", "nature", "client", "extra_fields")


admin.site.register(Member, MemberAdmin)
admin.site.register(Client, ClientAdmin)
admin.site.register(ClientScheduledStatus)
admin.site.register(Route)
admin.site.register(DeliveryHistory)
admin.site.register(Contact, ContactAdmin)
admin.site.register(Address)
admin.site.register(Relationship, RelationshipAdmin)
admin.site.register(Option)



import random

import factory

from souschef.billing.models import Billing


class BillingFactory(factory.DjangoModelFactory):
    class Meta:
        model = Billing

    total_amount = random.randrange(1, stop=75, step=1)
    billing_month = random.randrange(1, stop=12, step=1)
    billing_year = random.randrange(2016, stop=2020, step=1)
    detail = {}



from django.contrib import admin
from django.db.models.functions import Lower

from souschef.meal.models import (
    COMPONENT_GROUP_CHOICES,
    INGREDIENT_GROUP_CHOICES,
    RESTRICTED_ITEM_GROUP_CHOICES,
    Component,
    Component_ingredient,
    Incompatibility,
    Ingredient,
    Menu,
    Restricted_item,
)


class ComponentsInline(admin.TabularInline):
    model = Menu.components.through


class ComponentIngredientInline(admin.TabularInline):
    model = Component_ingredient


class IncompatibilityInline(admin.TabularInline):
    model = Incompatibility


class MenuAdmin(admin.ModelAdmin):
    """Allows accessing menu components within the Menu admin."""

    inlines = [ComponentsInline]


class ComponentAdmin(admin.ModelAdmin):
    """Allows accessing ingredients within the Component admin."""

    inlines = [ComponentIngredientInline]
    list_display = ("name", "component_group")
    search_fields = ("name",)

    def get_ordering(self, request):
        return [Lower("name")]

    def get_search_results(self, request, queryset, search_term):
        # search for group choices in their display value instead of DB value
        queryset, use_distinct = super().get_search_results(
            request, queryset, search_term
        )
        search_list = [
            choice[0]
            for choice in COMPONENT_GROUP_CHOICES
            if search_term.lower() in choice[1].lower()
        ]
        if search_list:
            queryset |= self.model.objects.filter(component_group__in=search_list)
        return queryset, use_distinct


class IngredientAdmin(admin.ModelAdmin):
    """Allows more control over the display of ingredients in the admin."""

    list_display = (
        "name",
        "ingredient_group",
    )
    search_fields = ("name",)

    def get_ordering(self, request):
        return [Lower("name")]

    def get_search_results(self, request, queryset, search_term):
        # search for group choices in their display value instead of DB value
        queryset, use_distinct = super().get_search_results(
            request, queryset, search_term
        )
        search_list = [
            choice[0]
            for choice in INGREDIENT_GROUP_CHOICES
            if search_term.lower() in choice[1].lower()
        ]
        if search_list:
            queryset |= self.model.objects.filter(ingredient_group__in=search_list)
        return queryset, use_distinct


class Restricted_itemAdmin(admin.ModelAdmin):
    """Allows accessing ingredients within the Restricted_item admin."""

    inlines = [IncompatibilityInline]
    list_display = (
        "name",
        "restricted_item_group",
    )
    search_fields = ("name",)

    def get_ordering(self, request):
        return [Lower("name")]

    def get_search_results(self, request, queryset, search_term):
        # search for group choices in their display value instead of DB value
        queryset, use_distinct = super().get_search_results(
            request, queryset, search_term
        )
        search_list = [
            choice[0]
            for choice in RESTRICTED_ITEM_GROUP_CHOICES
            if search_term.lower() in choice[1].lower()
        ]
        if search_list:
            queryset |= self.model.objects.filter(restricted_item_group__in=search_list)
        return queryset, use_distinct


admin.site.register(Component, ComponentAdmin)
admin.site.register(Restricted_item, Restricted_itemAdmin)
admin.site.register(Ingredient, IngredientAdmin)
admin.site.register(Menu, MenuAdmin)



import datetime
import os

import pkg_resources
from django.conf import settings

from souschef.member.models import (
    Client,
    Route,
)
from souschef.note.models import (
    Note,
    NoteFilter,
)
from souschef.order.models import (
    ORDER_STATUS_ORDERED,
    Order,
)


def get_sous_chef_version():
    if os.environ.get("CI") == "1":
        return "dev"

    return pkg_resources.require("souschef")[0].version


def total(request):
    """Passing entities total into RequestContext."""

    clients = Client.objects.count()
    orders = Order.objects.count()
    routes = Route.objects.count()
    # Only unread messages
    notes = Note.unread.count()

    COMMON_CONTEXT = {
        "CLIENTS_TOTAL": clients,
        "ORDERS_TOTAL": orders,
        "ROUTES_TOTAL": routes,
        "NOTES_TOTAL": notes,
        # This is pretty bad separation of concert to puth the following here
        # But since there is no actual view associated with menu.html (it's
        # just included is base.html, this is the most DRY way I found
        "CLIENT_FILTER_DEFAULT_STATUS": Client.ACTIVE,
        "ORDER_FILTER_DEFAULT_STATUS": ORDER_STATUS_ORDERED,
        "ORDER_FILTER_DEFAULT_DATE": datetime.datetime.now(),
        "NOTE_FILTER_DEFAULT_IS_READ": NoteFilter.NOTE_STATUS_UNREAD,
        "SC_ENVIRONMENT_NAME": settings.SOUSCHEF_ENVIRONMENT_NAME,
        "SC_VERSION": get_sous_chef_version(),
        "GIT_HEAD": settings.GIT_HEAD,
        "GIT_TAG": settings.GIT_TAG,
    }

    return COMMON_CONTEXT



# Generated by Django 1.10 on 2016-08-25 23:37

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("order", "0006_auto_20160803_2217"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="order_item",
            name="component",
        ),
    ]



# Generated by Django 1.10 on 2016-11-22 04:58

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0019_auto_20161111_1750"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="client",
            options={
                "ordering": ["-member__created_at"],
                "verbose_name_plural": "clients",
            },
        ),
    ]



import datetime

from django.contrib.auth.models import User
from django.test import TestCase
from django.urls import reverse

from souschef.billing.factories import BillingFactory
from souschef.billing.models import (
    Billing,
    calculate_amount_total,
)
from souschef.member.factories import ClientFactory
from souschef.order.factories import OrderFactory
from souschef.order.models import Order
from souschef.sous_chef.tests import TestMixin as SousChefTestMixin


class BillingTestCase(TestCase):
    fixtures = ["routes.json"]

    @classmethod
    def setUpTestData(cls):
        cls.client1 = ClientFactory()
        cls.billed_orders = OrderFactory.create_batch(
            10,
            delivery_date=datetime.datetime.today(),
            client=cls.client1,
            status="D",
        )
        cls.orders = OrderFactory.create_batch(
            10,
            delivery_date=datetime.datetime.today(),
            client=ClientFactory(),
            status="O",
        )

    def test_get_billable_orders(self):
        """
        Test that all the delivered orders for a given month are fetched.
        """
        month = datetime.datetime.now().strftime("%m")
        year = datetime.datetime.now().strftime("%Y")
        orders = Order.objects.get_billable_orders(year, month)
        self.assertEqual(len(self.billed_orders), orders.count())

    def testTotalAmount(self):
        total_amount = calculate_amount_total(self.orders)
        self.assertEqual(total_amount, 50)

    def testOrderDetail(self):
        pass

    def test_get_billable_orders_client(self):
        """
        Test that all the delivered orders for a given month and
        a given client are fetched.
        """
        month = datetime.datetime.now().strftime("%m")
        year = datetime.datetime.now().strftime("%Y")
        orders = Order.objects.get_billable_orders_client(month, year, self.client1)
        self.assertEqual(len(self.billed_orders), orders.count())


class BillingManagerTestCase(TestCase):
    fixtures = ["routes.json"]

    def setUp(self):
        self.today = datetime.datetime.today()
        self.billable_orders = OrderFactory.create_batch(
            10,
            delivery_date=self.today,
            status="D",
        )

    def test_billing_create_new(self):
        billing = Billing.objects.billing_create_new(self.today.year, self.today.month)
        self.assertEqual(10, billing.orders.all().count())
        self.assertEqual(self.today.month, billing.billing_month)
        self.assertEqual(self.today.year, billing.billing_year)
        # We created 10 orders, with one billable 10$ value item each
        self.assertEqual(50.00, billing.total_amount)

    def test_billing_get_period(self):
        billing = Billing.objects.billing_get_period(self.today.year, self.today.month)
        self.assertEqual(None, billing)


class RedirectAnonymousUserTestCase(SousChefTestMixin, TestCase):
    fixtures = ["routes.json"]

    def test_anonymous_user_gets_redirect_to_login_page(self):
        check = self.assertRedirectsWithAllMethods
        check(reverse("billing:list"))
        check(reverse("billing:create"))
        check(reverse("billing:add"))
        bill = BillingFactory()
        check(reverse("billing:view", kwargs={"pk": bill.id}))
        check(reverse("billing:delete", kwargs={"pk": bill.id}))


class BillingListViewTestCase(SousChefTestMixin, TestCase):
    def test_redirects_users_who_do_not_have_read_permission(self):
        # Setup
        User.objects.create_user(
            username="foo", email="foo@example.com", password="secure"
        )
        self.client.login(username="foo", password="secure")
        url = reverse("billing:list")
        # Run & check
        self.assertRedirectsWithAllMethods(url)

    def test_allow_access_to_users_with_read_permission(self):
        # Setup
        user = User.objects.create_user(
            username="foo", email="foo@example.com", password="secure"
        )
        user.is_staff = True
        user.save()
        self.client.login(username="foo", password="secure")
        url = reverse("billing:list")
        # Run
        response = self.client.get(url)
        # Check
        self.assertEqual(response.status_code, 200)


class BillingCreateViewTestCase(SousChefTestMixin, TestCase):
    def test_redirects_users_who_do_not_have_edit_permission(self):
        # Setup
        user = User.objects.create_user(
            username="foo", email="foo@example.com", password="secure"
        )
        user.is_staff = True
        user.save()
        self.client.login(username="foo", password="secure")
        url = reverse("billing:create")
        # Run & check
        self.assertRedirectsWithAllMethods(url)

    def test_allow_access_to_users_with_edit_permission(self):
        # Setup
        user = User.objects.create_superuser(
            username="foo", email="foo@example.com", password="secure"
        )
        user.save()
        self.client.login(username="foo", password="secure")
        url = reverse("billing:create")
        # Run
        response = self.client.get(url, {"delivery_date": ""}, follow=True)
        # Check
        self.assertEqual(response.status_code, 200)
        self.assertTrue("error message" in repr(response.content))
        response = self.client.get(url, {"delivery_date": "2016-1"}, follow=True)
        # Check
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.redirect_chain[-1][0], reverse("billing:list"))


class BillingAddViewTestCase(SousChefTestMixin, TestCase):
    def test_redirects_users_who_do_not_have_edit_permission(self):
        # Setup
        user = User.objects.create_user(
            username="foo", email="foo@example.com", password="secure"
        )
        user.is_staff = True
        user.save()
        self.client.login(username="foo", password="secure")
        url = reverse("billing:add")
        # Run & check
        self.assertRedirectsWithAllMethods(url)

    def test_allow_access_to_users_with_edit_permission(self):
        # Setup
        user = User.objects.create_superuser(
            username="foo", email="foo@example.com", password="secure"
        )
        user.save()
        self.client.login(username="foo", password="secure")
        url = reverse("billing:add")
        # Run
        response = self.client.get(url)
        # Check
        self.assertEqual(response.status_code, 200)


class BillingSummaryViewTestCase(SousChefTestMixin, TestCase):
    def test_redirects_users_who_do_not_have_read_permission(self):
        # Setup
        User.objects.create_user(
            username="foo", email="foo@example.com", password="secure"
        )
        self.client.login(username="foo", password="secure")
        bill = BillingFactory()
        url = reverse("billing:view", args=(bill.id,))
        # Run & check
        self.assertRedirectsWithAllMethods(url)

    def test_allow_access_to_users_with_read_permission(self):
        # Setup
        user = User.objects.create_user(
            username="foo", email="foo@example.com", password="secure"
        )
        user.is_staff = True
        user.save()
        self.client.login(username="foo", password="secure")
        bill = BillingFactory()
        url = reverse("billing:view", args=(bill.id,))
        # Run
        response = self.client.get(url)
        # Check
        self.assertEqual(response.status_code, 200)


class BillingDeleteViewTestCase(SousChefTestMixin, TestCase):
    def test_redirects_users_who_do_not_have_edit_permission(self):
        # Setup
        user = User.objects.create_user(
            username="foo", email="foo@example.com", password="secure"
        )
        user.is_staff = True
        user.save()
        self.client.login(username="foo", password="secure")
        bill = BillingFactory()
        url = reverse("billing:delete", args=(bill.id,))
        # Run & check
        self.assertRedirectsWithAllMethods(url)

    def test_allow_access_to_users_with_edit_permission(self):
        # Setup
        user = User.objects.create_superuser(
            username="foo", email="foo@example.com", password="secure"
        )
        user.save()
        self.client.login(username="foo", password="secure")
        bill = BillingFactory()
        url = reverse("billing:delete", args=(bill.id,))
        # Run
        response = self.client.post(url, {"next": "/"}, follow=True)
        # Check
        self.assertEqual(response.status_code, 200)



# Generated by Django 1.9.5 on 2016-06-23 01:00

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meal", "0002_ingredient_ingredient_group"),
        ("member", "0003_auto_20160615_2100"),
    ]

    operations = [
        migrations.CreateModel(
            name="Client_avoid_component",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Client_avoid_ingredient",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="client_option",
            name="value",
            field=models.CharField(max_length=100, null=True, verbose_name="value"),
        ),
        migrations.AlterField(
            model_name="client",
            name="billing_payment_type",
            field=models.CharField(
                choices=[
                    ("check", "Check"),
                    ("cash", "Cash"),
                    ("debit", "Debit card"),
                    ("credit", "Credit card"),
                    ("eft", "EFT"),
                ],
                max_length=10,
                null=True,
                verbose_name="Payment Type",
            ),
        ),
        migrations.AlterField(
            model_name="client",
            name="gender",
            field=models.CharField(
                blank=True,
                choices=[("", "Gender"), ("F", "Female"), ("M", "Male")],
                default="U",
                max_length=1,
                null="True",
            ),
        ),
        migrations.AlterField(
            model_name="client",
            name="route",
            field=models.ForeignKey(
                blank=True,
                default="",
                on_delete=django.db.models.deletion.CASCADE,
                to="member.Route",
                verbose_name="route",
            ),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="option",
            name="option_group",
            field=models.CharField(
                choices=[
                    ("main dish size", "Main dish size"),
                    ("dish", "Dish"),
                    ("preparation", "Preparation"),
                    ("other order item", "Other order item"),
                ],
                max_length=100,
                verbose_name="option group",
            ),
        ),
        migrations.AddField(
            model_name="client_avoid_ingredient",
            name="client",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="member.Client",
                verbose_name="client",
            ),
        ),
        migrations.AddField(
            model_name="client_avoid_ingredient",
            name="ingredient",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="meal.Ingredient",
                verbose_name="ingredient",
            ),
        ),
        migrations.AddField(
            model_name="client_avoid_component",
            name="client",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="member.Client",
                verbose_name="client",
            ),
        ),
        migrations.AddField(
            model_name="client_avoid_component",
            name="component",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="meal.Component",
                verbose_name="component",
            ),
        ),
    ]



from django import template

register = template.Library()


@register.filter(name="get_item")
def get_item(o, key):
    try:
        return o[key]
    except Exception:
        return None


@register.filter(name="alter_field_class")
def alter_field_class(field, css):
    return field.as_widget(attrs={"class": css})



# Generated by Django 1.11.29 on 2020-10-30 19:40

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("billing", "0003_auto_20161122_0515"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="billing",
            options={"ordering": ["-billing_year", "-billing_month"]},
        ),
    ]



# Generated by Django 1.9.5 on 2016-07-21 11:20

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meal", "0003_fix224a"),
    ]

    operations = [
        migrations.AddField(
            model_name="component_ingredient",
            name="date",
            field=models.DateField(blank=True, null=True, verbose_name="date"),
        ),
    ]



# Generated by Django 1.10 on 2017-01-15 04:28

import django.db.models.deletion
from django.db import migrations, models


def migrate_old_data(apps, schema_editor):
    # set for each product the correct language
    NotePriority = apps.get_model("note", "NotePriority")
    Note = apps.get_model("note", "Note")

    normal_priority = NotePriority.objects.get(name="Normal")
    urgent_priority = NotePriority.objects.get(name="Urgent")

    for note in Note.objects.all():
        if note.priority == normal_priority.name.lower():
            note.priority_temp = normal_priority
            note.save()
        elif note.priority == urgent_priority.name.lower():
            note.priority_temp = urgent_priority
            note.save()


class Migration(migrations.Migration):
    dependencies = [
        ("note", "0004_notepriority"),
    ]

    operations = [
        migrations.AddField(
            model_name="note",
            name="priority_temp",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="notes",
                to="note.NotePriority",
                verbose_name="Priority",
            ),
        ),
        migrations.RunPython(migrate_old_data),
    ]



# Generated by Django 1.10 on 2016-11-22 04:58

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("billing", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="billing",
            name="total_amount",
            field=models.DecimalField(
                decimal_places=2, max_digits=8, verbose_name="total_amount"
            ),
        ),
    ]



# Generated by Django 1.11.29 on 2021-07-23 15:29

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("note", "0013_date_modified"),
    ]

    operations = [
        migrations.AlterField(
            model_name="note",
            name="date_created",
            field=models.DateTimeField(
                default=django.utils.timezone.now, verbose_name="Date Created"
            ),
        ),
    ]



import re

from django import forms
from django.utils.encoding import smart_text
from django.utils.translation import ugettext_lazy as _
from localflavor.ca.forms import CAPhoneNumberField


class CAPhoneNumberExtField(CAPhoneNumberField):
    """Canadian phone number form field."""

    phone_digits_re = re.compile(r"^(?:1-?)?(\d{3})[-\.]?(\d{3})[-\.]?(\d{4})$")
    phone_digits_with_ext = re.compile(
        r"^(?:1-?)?(\d{3})[-\.]?(\d{3})[-\.]?(\d{4})#?(\d*)$"
    )

    default_error_messages = {
        "invalid": _("Phone numbers must be in XXX-XXX-XXXX # XXXXX format."),
    }

    def clean(self, value):
        try:
            return super().clean(value)
        except forms.ValidationError as error:
            value = re.sub(r"(\(|\)|\s+)", "", smart_text(value))
            m = self.phone_digits_with_ext.search(value)
            if m:
                return f"{m.group(1)}-{m.group(2)}-{m.group(3)} #{m.group(4)}"
            raise error



# Generated by Django 1.10 on 2016-08-26 00:07

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meal", "0005_fix241b"),
    ]

    operations = [
        migrations.AddField(
            model_name="menu",
            name="components",
            field=models.ManyToManyField(
                through="meal.Menu_component", to="meal.Component"
            ),
        ),
        migrations.AddField(
            model_name="restricted_item",
            name="ingredients",
            field=models.ManyToManyField(
                through="meal.Incompatibility", to="meal.Ingredient"
            ),
        ),
    ]



# Generated by Django 1.9.8 on 2016-08-05 17:57

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0010_auto_20160803_2052"),
    ]

    operations = [
        migrations.AddField(
            model_name="client",
            name="delivery_note",
            field=models.TextField(blank=True, null=True, verbose_name="Delivery Note"),
        ),
    ]
